local LrErrors = import 'LrErrors'
local LrHttp = import 'LrHttp'
local LrMD5 = import 'LrMD5'
local LrPath = import 'LrPathUtils'
local LrStringUtils = import 'LrStringUtils'
local LrTasks = import 'LrTasks'

local LrLogger = import 'LrLogger'
local logger = LrLogger("ttg")
logger:enable( "logfile" )

local version = '4.0.0'

JSON = (assert(loadfile(LrPath.child(_PLUGIN.path, "JSON.lua"))))()

function on_error(headers, error_message, action, result, log_message, throw_error)

	if headers and (headers.error or not (headers.status==200 or headers.status==201)) then

		if headers.status then
			if headers.status == 404 then
				error_message = "Unable to find Publisher.\nPlease check that the API URL is correct."
				if throw_error then
					LrErrors.throwUserError(error_message)
				else
					return error_message
				end
			elseif headers.status == 401 then
				error_message = "Publisher received a 401 Unauthorized error from your server.\n\nIf you are using Basic authentication on your server, add your username and password to the HTTP Authentication settings of the Publisher instance."
				if throw_error then
					LrErrors.throwUserError(error_message)
				else
					return error_message
				end
			elseif headers.status == 500 then
				local decoded_result = (JSON:decode(result))

				if decoded_result == nil then
					error_message = error_message.." (Code: "..headers.status
					if headers.error ~= nil then
						error_message = error_message..", Error: "..TTGUtils.dumpvar(headers.error)
					end

					error_message = error_message..")"
				else
					if decoded_result.status == nil then
						error_message = 'Unable to perform action: '..action..'. \n\nServer response:\n\n'..result
					elseif decoded_result.status ~= 'success' then
						if decoded_result.message == nil then
							error_message = 'Unable to perform action: '..action..'.  No message was provided. \n\nServer response:\n\n'..result
						else
							error_message = 'Unable to perform action: '..action..'\n\nMessage: '..decoded_result.message
						end
					end
				end
			else
				error_message = error_message.." (Code: "..headers.status
				if headers.error ~= nil then
					error_message = error_message..", Error: "..TTGUtils.dumpvar(headers.error)
				end
					
				error_message = error_message..")"
			end
		elseif headers.error and headers.error.errorCode and headers.error.errorCode == 'cannotConnectToHost' then
			error_message = "Could not connect to network.  You need to be online to use TTG Publisher."
		else
			error_message = error_message..'\n\nUnable to complete the action.\n\n  Headers: '..TTGUtils.dumpvar(headers)
		end
		
		if log_message and error_message ~= nil and error_message ~= ''then
			logger:debug(error_message)
		end
		
		if throw_error then
			LrErrors.throwUserError(error_message)
		else
			return error_message
		end
	end	
	
	return nil
end

function decodeResult(action, result)
	
	if result == nil or result == '' or result == 'null' then
		LrErrors.throwUserError('Unable to perform action: '..action..'. No response was returned.')
	end
	
	if result:match('^%s*<') then
		if action == 'checkAuth' then
			LrErrors.throwUserError('It appears that the API URL does not point to the publisher API.  Ensure that the URL finishes with \'publisher/\'.')
		else
		end	
			LrErrors.throwUserError('Unable to perform action: '..action..'. \n\nThe server did not respond in the correct format. \n\nServer response:\n\n'..result)
	end
	
	local decoded_result = (JSON:decode(result))

	if decoded_result == nil then
		LrErrors.throwUserError('Unable to perform action: '..action..'. \n\nServer response:\n\n'..result)
	end

	if decoded_result.status == nil then
		LrErrors.throwUserError('Unable to perform action: '..action..'. \n\nServer response:\n\n'..result)
	elseif decoded_result.status ~= 'success' then
		if decoded_result.message == nil then
			LrErrors.throwUserError('Unable to perform action: '..action..'.  No message was provided. \n\nServer response:\n\n'..result)
		else
			LrErrors.throwUserError('Unable to perform action: '..action..'\n\n'..decoded_result.message)
		end
	end

	return decoded_result
end


TTGApi = {}

function TTGApi.apiUrl(propertyTable)
	local delim_to = strrpos(propertyTable.apiURL, 'm=publisher', 0)
	if delim_to > 1 then
		return propertyTable.apiURL..'&source=lightroom'
	else
		return propertyTable.apiURL..'?source=lightroom'
	end

	-- Old format: http://yoursite.com/backlight/publisher/?source=lightroom
	-- New format: http://yoursite.com/backlight/?m=publisher&source=lightroom
	-- return string.gsub(propertyTable.apiURL, "backlight/publisher/$", "backlight/?m=publisher&source=lightroom")
end

function TTGApi.prepareParameters(propertyTable, parameters)
	-- Encode HTML entities, e.g. & => &amp;
	parameters.version = version
	parameters = TTGUtils.encodeTable(parameters, {blockContent = true, sidebarCopy = true, tray2Copy = true})
	parameters = JSON:encode(parameters)
	local cs = LrMD5.digest(propertyTable.apiKey..parameters)

	-- URL encode, e.g. '&' to '%26'.
	-- Must be done after creating cs, as the PHP backend sees URL-encoded characters as they were before encoding.  

	return parameters,cs
end

function TTGApi.callRemoteAPI(propertyTable, parameters)
	local action = parameters.action
	parameters.galleryName = propertyTable.galleryName
	local parameters,cs = TTGApi.prepareParameters(propertyTable, parameters)
	parameters = TTGUtils.urlEncode(parameters) -- only do this for non-multi-part calls

	local p2 = "parameters="..parameters.."&cs="..cs
	local url = TTGApi.apiUrl(propertyTable)

	logger:debug("callRemoteAPI: "..url..'&'..p2)
	
	local headers = {
		{field = 'X-Gallery-Request-Method', value = "post" },
		{field = 'Content-Type', value = "application/x-www-form-urlencoded" }, 
		{field = 'Content-Length', value = #p2 },
		{field = 'Referer', value = url },
	}

	if propertyTable.httpUsername ~= nil and propertyTable.httpUsername ~= '' and propertyTable.httpPassword ~= nil and propertyTable.httpPassword ~= '' then
		table.insert(headers,  {field = 'Authorization', value = 'Basic '..LrStringUtils.encodeBase64(propertyTable.httpUsername..':'..propertyTable.httpPassword)})
	end

	--[[
	local debug_hdrs = "headers: "
	for k,v in pairs(hdrs) do
		if(type(v)=="table") then
			debug_hdrs = debug_hdrs..k.."=>["
			for kk,vv in pairs(v) do
				debug_hdrs = debug_hdrs..kk.."=>"..tostring(vv)
			end			
		else
			debug_hdrs = debug_hdrs..k.."=>"..tostring(v)..", "
		end
	end
	
	logger:debug(debug_hdrs)
	]]--

	local result, hdrs = '', ''
	local attempts = 0
	local success = false
	local error_message = nil

	repeat
		result, hdrs = LrHttp.post( TTGApi.apiUrl(propertyTable), p2, headers )
		error_message = on_error(hdrs, "Error performing function.", action, result, false, false)

		if error_message ~= nil then
			success = false
			attempts = attempts+1
		else
			success = true
		end

		if propertyTable.delayAPIRequests and action == 'saveMetadata' then
			LrTasks.sleep(propertyTable.delayAPIRequestsSeconds)
		elseif not success then
			LrTasks.sleep(3) -- don't slam the server on failure
		end
	until attempts >= 5 or success

	if success == false and error_message ~= nil then
		logger:debug('Throwing error_message: '..error_message)
		LrErrors.throwUserError(error_message)
	end
	
	if result == nil then
		logger:debug("callRemoteAPI failed")
		return nil
	end
	
	logger:debug("callRemoteAPI result: "..result)
	
	local decoded_result = decodeResult(action, result)

	if decoded_result.version ~= nil then
		local prefs = import 'LrPrefs'.prefsForPlugin()
		-- local inspected = TTGUtils.inspect(prefs)
		-- logger:debug('callRemoteAPI prefs: '..inspected)
		prefs.serverVersion = decoded_result.version

		-- How can I write to this, so that it sticks?!
		-- logger:debug('server version: '..decoded_result.version)
		propertyTable.serverVersion = decoded_result.version
	end

	return decoded_result
end

function TTGApi.callRemoteMultiPartAPI(propertyTable, parameters, fileName, filePath)

	local action = parameters.action
	parameters.galleryName = propertyTable.galleryName
	local parameters,cs = TTGApi.prepareParameters(propertyTable, parameters)

	local headers = {
		{field = 'X-TTG-Request-Method', value = "post" },
	}

	if propertyTable.httpUsername ~= nil and propertyTable.httpUsername ~= '' and propertyTable.httpPassword ~= nil and propertyTable.httpPassword ~= '' then
		table.insert(headers,  {field = 'Authorization', value = 'Basic '..LrStringUtils.encodeBase64(propertyTable.httpUsername..':'..propertyTable.httpPassword)})
	end

	local mimeChunks = {
		{ name = "parameters", value = parameters, contentType = "application/json" },
		{ name = "cs", value = cs, contentType = "text/plain" },
		{ name = 'file', fileName = fileName, filePath = filePath, contentType = 'application/octet-stream' }
	}

	local targetUrl = TTGApi.apiUrl(propertyTable)
	logger:debug("UploadRendition: url="..targetUrl.."; parameters="..parameters)

	local result, hdrs = '', ''
	local attempts = 0
	local success = false
	local error_message = nil

	repeat
		result, hdrs = LrHttp.postMultipart( targetUrl, mimeChunks, headers )

		error_message = on_error(hdrs, "Error uploading rendition.", action, result, false, false)

		if error_message ~= nil then
			success = false
			attempts = attempts+1
		else
			success = true
		end

		if propertyTable.delayAPIRequests then
			LrTasks.sleep(propertyTable.delayAPIRequestsSeconds)
		elseif not success then
			LrTasks.sleep(3) -- don't slam the server on failure
		end
	until attempts >= 5 or success

	if success == false and error_message ~= nil then
		logger:debug('Throwing error_message: '..error_message)
		LrErrors.throwUserError(error_message)
	end

	logger:debug("result: '"..tostring(result).."'")

	local decoded_result = decodeResult(action, result)
	return decoded_result
end

function TTGApi.checkAuth(propertyTable)

	local parameters = {action = 'checkAuth'}
	
	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
	return decoded_result.result,decoded_result.message
end

function TTGApi.fetchTemplates(propertyTable, id)

	local parameters = {action = 'listTemplates' }

	if id ~= nil then
		parameters.id = id
	end

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
	local templates = {}
	local cartPricingSchemes = {}
	local crgFeedbackProfiles = {}
	
	if decoded_result.templates == nil then
		LrErrors.throwUserError('templates were not loaded')
	end

	for i,line in ipairs(decoded_result.templates) do
		local lineValue = nil

		if line.id ~= nil then
		   lineValue = line.id
		else
			lineValue = line.title
		end

		templates[#templates+1] = {title=line.title, value=lineValue, type=line.type}
	end

	local selectedTemplate = '';

	if decoded_result.selectedTemplate ~= nil then
		selectedTemplate = decoded_result.selectedTemplate
	end

	if decoded_result.cartPricingSchemes ~= nil then
		for i,line in ipairs(decoded_result.cartPricingSchemes) do
			cartPricingSchemes[#cartPricingSchemes+1] = {title=line.title, value=line.title, id=line.id}
		end
	end
	
	if decoded_result.crgFeedbackProfiles ~= nil then
		for i,line in ipairs(decoded_result.crgFeedbackProfiles) do
			crgFeedbackProfiles[#crgFeedbackProfiles+1] = {title=line.title, value=line.id}
		end
	end

	logger:debug("selectedTemplate: "..selectedTemplate)
	logger:debug("crgProfiles: "..TTGUtils.inspect(crgProfiles))

	return templates, selectedTemplate, cartPricingSchemes, crgFeedbackProfiles;
end

function TTGApi.fetchAutoindexTemplates(propertyTable, id)

	local parameters = {action = 'listAutoindexTemplates'}

	if id ~= nil then
		parameters.id = id
	end

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
	local templates = {}
	
	if decoded_result.templates ~= nil then
		for i,line in ipairs(decoded_result.templates) do
			local lineValue = nil

			if line.id ~= nil then
				lineValue = line.id
			else
				lineValue = line.title
			end

			templates[#templates+1] = {title=line.title, value=lineValue} --, type=line.type}
		end
	end

	local selectedTemplate = '';

	if decoded_result.selectedTemplate ~= nil then
		selectedTemplate = decoded_result.selectedTemplate
	end

	return templates, selectedTemplate;
end

function TTGApi.getSetupForTemplate(propertyTable, remoteId, templateType, template)

	local parameters = {action = 'getSetupForTemplate'}
	parameters.templateType = templateType
	parameters.template = template

	if remoteId ~= nil then
		parameters.id = remoteId
	end

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)



	if decoded_result.setup ~= nil then
		local setup = decoded_result.setup
		local album = decoded_result.album
		local thumbnails = decoded_result.thumbnails

		local serverVersion = '';

		if decoded_result.version ~= nil then
			serverVersion = decoded_result.version
		end

		return setup, album, thumbnails, serverVersion
	else
		return decoded_result
	end
end

function TTGApi.albumExists(propertyTable, parentId, slug)

	local parameters = {}
	parameters.action = "checkAlbum"
	parameters.slug = slug
	parameters.parentId = parentId

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
	return decoded_result.result
end

function TTGApi.getAlbum(propertyTable, id)

	local parameters = {}
	parameters.action = "getAlbum"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	return decoded_result
end

function TTGApi.getPublishedRenditions(propertyTable, id)

	local parameters = {}
	parameters.action = "getPublishedRenditions"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	return decoded_result.photos
end

function TTGApi.getThumbnails(propertyTable, id)

	local parameters = {}
	parameters.action = "getThumbnails"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	return decoded_result.thumbnails
end

function TTGApi.getPhotosForHeroImage(propertyTable, id)

	local parameters = {}
	parameters.action = "getPhotosForHeroImage"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	return decoded_result.photos
end

function TTGApi.createAutoSlug(propertyTable, slug)

	local parameters = {}
	parameters.action = "createAutoSlug"
	parameters.slug = slug

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
	return decoded_result.slug
end

function TTGApi.createAlbum(propertyTable, parameters)

	parameters.action = "createAlbum"
	
	-- TTGUtils.rftabledump(parameters)
	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local id = decoded_result.id
	local url = decoded_result.url
	local slug = decoded_result.slug

	return id,url,slug
end

function TTGApi.updateAlbum(propertyTable, parameters)

	parameters.action = "updateAlbum"
	
	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local id = decoded_result.id
	local url = decoded_result.url
	local slug = decoded_result.slug

	return id,url,slug
end

function TTGApi.createAlbumSet(propertyTable, parameters)

	parameters.action = "createAlbumSet"
	
	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local id = decoded_result.id
	local url = decoded_result.url
	local slug = decoded_result.slug

	return id,url,slug
end

function TTGApi.updateAlbumSet(propertyTable, parameters)

	parameters.action = "updateAlbumSet"
	
	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local id = decoded_result.id
	local url = decoded_result.url
	local slug = decoded_result.slug

	return id,url,slug
end

function TTGApi.reparent(propertyTable, id, parentId)

	local parameters = {}
	parameters.action = "reparent"
	parameters.id = id
	parameters.parentId = parentId

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local url = decoded_result.url
	local id = decoded_result.id

	return id,url
end

function TTGApi.setPhotoOrder(id, remoteIdSequence, propertyTable)

	local parameters = {}
	--parameters.name = name
	parameters.action = "setPhotoOrder"
	parameters.id = id
	parameters.idSequence = remoteIdSequence

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	
end

function TTGApi.cleanupPartialUploads(id, renditionPaths, propertyTable)

	local parameters = {}
	parameters.action = "cleanupPartialUploads"
	parameters.id = id
	parameters.renditionPaths = renditionPaths

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

end

function TTGApi.resetHasUserOrder(id, propertyTable)

	local parameters = {}
	parameters.action = "resetHasUserOrder"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

end

function TTGApi.deleteAlbum(propertyTable, id)
	local parameters = {}
	parameters.action = "deleteAlbum"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	local url = decoded_result.url
	local id = decoded_result.id

	return id

end

function TTGApi.uploadRendition(id, photoId, filePath, fileName, parameters, propertyTable)
	local targetUrl = TTGApi.apiUrl(propertyTable)
	
	parameters.action="uploadRendition"
	if fileName and fileName~="" then parameters.name = fileName end
	parameters.id=id
	parameters.photoId=photoId
	parameters.template='highslide_php_plugins_paged'

	local decoded_result = TTGApi.callRemoteMultiPartAPI(propertyTable, parameters, fileName, filePath)

	local id = decoded_result.asset.id
	local webUrl = decoded_result.asset.url

	logger:debug("uploadRendition finished with id: "..id)

	return id,webUrl
end

function TTGApi.saveMetadata(id, photoId, metadata, propertyTable)
	local parameters = {}
	parameters.action="saveMetadata"
	parameters.id=id
	parameters.photoId=photoId

	local count=0

	for k,v in pairs(metadata) do 
		count = count+1
		parameters['metadata_name_'..count] = k
		parameters['metadata_value_'..count] = tostring(v)
	end

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

end

function TTGApi.deletePhoto(propertyTable, id)

	local parameters = {}
	parameters.action = "deletePhoto"
	parameters.id = id

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)

	return true
end

function TTGApi.migrateGalleries(topLevelGalleryDirectory, idsOnService, propertyTable)

	local parameters = {}
	parameters.action = "migrateGalleries"
	parameters.id = id
	parameters.topLevelGalleryDirectory = topLevelGalleryDirectory
	parameters.idsOnService = idsOnService

	local decoded_result = TTGApi.callRemoteAPI(propertyTable, parameters)
	local migrated_album_ids = decoded_result.migrated_album_ids
	
	return migrated_album_ids
end
