require 'TTGMigrator'
require 'TTGUtils'

local LrBinding = import 'LrBinding'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrFunctionContext = import 'LrFunctionContext'
local LrTasks = import 'LrTasks'
local LrView = import 'LrView'

local bind = LrView.bind
local share = LrView.share
local prefs = import 'LrPrefs'.prefsForPlugin()

local LrLogger = import 'LrLogger'
local logger = LrLogger("ttg")
logger:enable("logfile")

TTGDialogs = {}

local itemsPerPage = { { title = "All in one page", value = 0 } }

for i = 1, 250, 1 do
	table.insert(itemsPerPage, { title = i, value = i })
end

for i = 300, 500, 50 do
	table.insert(itemsPerPage, { title = i, value = i })
end

TTGDialogs.itemsPerPage = itemsPerPage

local delaySeconds = {}

for i = 0.5, 9.5, 0.5 do
	table.insert(delaySeconds, { title = string.format("%3.1f", i), value = i })
end

for i = 10, 30, 1 do
	table.insert(delaySeconds, { title = i, value = i })
	--table.insert(delaySeconds, { title = i, value = i })
end

for i = 35, 60, 5 do
	table.insert(delaySeconds, { title = i, value = i })
	--table.insert(delaySeconds, { title = i, value = i })
end

TTGDialogs.delaySeconds = delaySeconds

TTGDialogs.ordering = {
	{ title = "Managed by Lightroom", value = "lightroom" },
	{ title = "Order by File Name", value = "filename" },
	{ title = "Order by File Name - descending", value = "filename_desc" },
	{ title = "Order by Capture Time", value = "capture_time" },
	{ title = "Order by Capture Time - descending", value = "capture_time_desc" },
	{ title = "Custom Order", value = "custom" }
}

-- Needs to be the same as returned by PublisherRootDAO::getAlbumSetAlbumOrderingLookup()
TTGDialogs.albumSetOrdering = {
	{ title = "Order by Album Title", value = "title" },
	{ title = "Order by Album Title - descending", value = "title_desc" },
	{ title = "Order by Album Slug", value = "slug" },
	{ title = "Order by Album Slug - descending", value = "slug_desc" },
	{ title = "Order by Album Creation Time", value = "created" },
	{ title = "Order by Album Creation Time - descending", value = "created_desc" },
	{ title = "Order by Album Modification Time", value = "modified" },
	{ title = "Order by Album Modification Time - descending", value = "modified_desc" }
}

TTGDialogs.pageCopyOptions = { { title = "Replace template Page Copy", value = "replace" }, { title = "Position new copy above existing Page Copy", value = "above" }, { title = "Position new copy below existing Page Copy", value = "below" } }
TTGDialogs.sidebarCopyOptions = { { title = "Replace template Sidebar Copy", value = "replace" }, { title = "Position new copy above existing Sidebar Copy", value = "above" }, { title = "Position new copy below existing Sidebar Copy", value = "below" } }
TTGDialogs.tray1CopyOptions = { { title = "Replace template Pallet 1 Copy", value = "replace" }, { title = "Position new copy above existing Pallet 1 Copy", value = "above" }, { title = "Position new copy below existing Tray 1 Copy", value = "below" } }
TTGDialogs.tray2CopyOptions = { { title = "Replace template Pallet 2 Copy", value = "replace" }, { title = "Position new copy above existing Pallet 2 Copy", value = "above" }, { title = "Position new copy below existing Tray 2 Copy", value = "below" } }
TTGDialogs.crgClientModes = { { title = "Managed by Client Response Admin", value = "managed" }, { title = "Unmanaged", value = "standalone" } }
TTGDialogs.visibilityOptions = { { title = "Public - visible to everyone", value = "public" }, { title = "Protected - requires Access Code", value = "protected" }, { title = "Private - requires Client account and access", value = "private" } }

function check_auth(propertyTable)
	local catalog = (import 'LrApplication').activeCatalog()
	local inspected = TTGUtils.inspect(propertyTable)
	--logger:debug("inspected: "..inspected)

	LrTasks.startAsyncTask(function()
		local result, message = TTGApi.checkAuth(propertyTable)
		logger:debug('result: ' .. result)
		if result == 'authenticated' then
			if propertyTable.apiKey == 'ttg1234' or propertyTable.apiKey == 'ttg8888' or propertyTable.apiKey == 'abc1234' then
				LrDialogs.message("Authentication Check", "The authentication check was successful, however you have set Publisher to use the default password.\n\nFor security, set a strong API Key by editing the file publisher/config.php on your server and update the API Key in your Lightroom Publisher settings.", "info")
			elseif propertyTable.apiKey == '' then
				LrDialogs.message("Authentication Check", "The authentication check was successful, however you have set Publisher to use a blank password.\n\nFor secrurity, set a strong API Key by editing the file publisher/config.php on your server and update the API Key in your Lightroom Publisher settings.", "info")
			elseif message ~= nil then
				LrDialogs.message("Authentication Check", "The authentication check was successful, however publisher found the following problem:\n\n" .. message, "info")
			else
				LrDialogs.message("Authentication Check", "The authentication check was successful", "info")
			end
		else
			LrDialogs.message("Authentication Check", "The authentication check was NOT successful.  Please check the URL and API Key.", "info")
		end
	end)
end

function copy_album_settings(importCollectionSettings, collectionSettings)
	local inspected = TTGUtils.inspect(importCollectionSettings)

	--logger:debug('inspected: '..inspected)

	for k, v in pairs(importCollectionSettings) do
		if k ~= 'LR_liveName' and k ~= 'title' and k ~= 'slug' and k ~= 'breadcrumb' then
			collectionSettings[k] = v
		end
	end

	if importCollectionSettings.contentBlock == nil then
		collectionSettings.contentBlock = ''
	end

	if importCollectionSettings.sidebarCopy == nil then
		collectionSettings.sidebarCopy = ''
	end

	if importCollectionSettings.tray2Copy == nil then
		collectionSettings.tray2Copy = ''
	end
end

function import_album_set(idHelper, info, album_id)
	local collectionSettings = assert(info.collectionSettings)
	local service = assert(info.publishService)

	local importCollectionSettings = idHelper:findAlbumSetById(album_id)

	if importCollectionSettings ~= nil then
		logger:debug('found album set for id: ' .. album_id)
		copy_album_settings(importCollectionSettings, collectionSettings)

		local catalog = (import 'LrApplication').activeCatalog()

		catalog:withWriteAccessDo("albumSetDefaults", function(context)
			catalog:setPropertyForPlugin(_PLUGIN, service:getName() .. '-lastSelectedAlbumSet', album_id)
		end, writeTimeout)
	else
		logger:debug('coult not find album set with id: ' .. album_id)
	end
end

function import_album(idHelper, info, album_id)
	local collectionSettings = assert(info.collectionSettings)
	local service = assert(info.publishService)

	local importCollectionSettings = idHelper:findAlbumById(album_id)

	if importCollectionSettings ~= nil then
		logger:debug('found album for id: ' .. album_id)
		copy_album_settings(importCollectionSettings, collectionSettings)

		local catalog = (import 'LrApplication').activeCatalog()

		catalog:withWriteAccessDo("albumDefaults", function(context)
			catalog:setPropertyForPlugin(_PLUGIN, service:getName() .. '-lastSelectedAlbum', album_id)
		end, writeTimeout)
	else
		logger:debug('coult not find album with id: ' .. album_id)
	end
end

function createSettingsTable(f, collectionSettings)
	return {
		bind_to_object = collectionSettings, -- all controls bound to our table
		spacing = f:control_spacing(), -- default spacing for the child rows
	}
end

function TTGDialogs.bottomSections(f, propertyTable)

	local sections = {}
	return sections
end

function TTGDialogs.topSections(f, propertyTable)

	local f = LrView.osFactory()

	local sections = {}
	propertyTable.publishThumbnailsOnlyEnabled = not propertyTable.pushMetadataOnly

	propertyTable:addObserver( 'pushMetadataOnly', function( properties, key, newValue )
		propertyTable.publishThumbnailsOnlyEnabled = not propertyTable.pushMetadataOnly
	end )

	propertyTable.delayAPIRequestsSecondsEnabled = propertyTable.delayAPIRequests
	propertyTable:addObserver( 'delayAPIRequests', function( properties, key, newValue )
		propertyTable.delayAPIRequestsSecondsEnabled = propertyTable.delayAPIRequests
	end )

	if propertyTable.LR_isExportForPublish then
		table.insert(sections, {
			title = "Publisher API Details",
			synopsis = "The API details to use for Publisher",

			f:row {
				spacing = f:label_spacing(),

				f:static_text {
					title = "API URL",
					alignment = 'right',
					width = share 'title_width',
				},

				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 37,
					value = bind 'apiURL',
					validate = function(view, value)
						if value ~= nil then
							return true, TTGUtils.addTrailingSlashIfNeeded(value)
						end
					end
				}
			},
			f:row {
				spacing = f:label_spacing(),

				f:static_text {
					title = "API Key",
					alignment = 'right',
					width = share 'title_width',
				},

				f:password_field {
					bind_to_object = apiURL,
					fill_horizonal = 1,
					width_in_chars = 37,
					value = bind 'apiKey',
				},
				f:spacer {
					width = 5
				},
				f:push_button {
					title = "Check Authentication",
					action = function()
						check_auth(propertyTable)
					end
				}
			}
		})

		table.insert(sections, {
			title = "Other Publisher Options",
			synopsis = "Other options",

			f:row {
				spacing = f:label_spacing(),

				f:static_text {
					title = "Top-level galleries directory",
					alignment = 'right',
					width = share 'atitle_width',
				},

				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 20,
					value = bind 'galleryName',
					validate = function(view, value)
						if value ~= nil then
							return true, TTGUtils.createSlugFromString(value, { default = 'galleries', allowSlash = true, preserve = true })
						end
					end
				}
			},

			f:row {
				spacing = f:control_spacing(),

				f:checkbox {
					title = "Show albums in browser after publishing",
					value = bind { key = "openAlbumAfterExport" }
				},
			},

			f:row {
				spacing = f:control_spacing(),

				f:checkbox {
					title = "Preserve capitalization in album slugs",
					value = bind { key = "preserveCapitalizationInAlbumSlugs" }
				},
			},

			f:row {
				spacing = f:control_spacing(),

				f:checkbox {
					title = "Push metadata without updating existing photos",
					value = bind { key = "pushMetadataOnly" }
				},
			},

			f:row {
				spacing = f:control_spacing(),

				f:checkbox {
					title = "Publish thumbnails without updating other renditions",
					value = bind { key = "publishThumbnailsOnly" },
					enabled = bind 'publishThumbnailsOnlyEnabled',
					-- how to negate this for binding??
					-- enabled = bind 'pushMetadataOnly',
				},
			},

			f:row {
				f:checkbox {
					title = "Delay API Requests by",
					value = bind { key = "delayAPIRequests" },
				},
				f:popup_menu {
					value = bind 'delayAPIRequestsSeconds',
					enabled = bind 'delayAPIRequestsSecondsEnabled',
					items = TTGDialogs.delaySeconds,
					--fill_horizontal = 1
				},
				f:static_text {
					title = "seconds",
					alignment = 'left',
				}
			},
		})

		table.insert(sections, {
			title = "HTTP Authentication",
			synopsis = "HTTP Authentication",

			f:row {
				--[[f:spacer {
					width = share 'title_width'
				},]] --
				f:static_text {
					title = "Complete these fields for sites protected by HTTP Basic Authentication. Otherwise, leave empty.",
					alignment = 'left',
					size = 'small',
					--width = 400,
				},
			},

			f:row {
				spacing = f:label_spacing(),

				f:static_text {
					title = "Username",
					alignment = 'right',
					width = share 'username_width',
				},

				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 20,
					value = bind 'httpUsername',
				}
			},

			f:row {
				spacing = f:label_spacing(),

				f:static_text {
					title = "Password",
					alignment = 'right',
					width = share 'username_width',
				},

				f:password_field {
					fill_horizonal = 1,
					width_in_chars = 20,
					value = bind 'httpPassword',
				}
			},
		})
	end

	return sections
end

function TTGDialogs.albumSetSettingsDialog(f, publishSettings, info)

	--local publishSettings = info.publishService:getPublishSettings()
	local collectionSettings = assert(info.collectionSettings)

	local service = assert(info.publishService)
	local idHelper = TTGGalleryIDHelper.new({ service = service })
	local albumSetsForPublisherService = idHelper:gatherAlbumSetsForPublisherService()

	local base_table = createSettingsTable(f, collectionSettings)
	local feature_table = createSettingsTable(f, collectionSettings)
	local page_copy_table = createSettingsTable(f, collectionSettings)

	local importTable = LrFunctionContext.callWithContext('albumSetSettingsDialog', function(context)
		return LrBinding.makePropertyTable(context)
	end)

	local catalog = (import 'LrApplication').activeCatalog()

	local lastSelectedAlbumSet = catalog:getPropertyForPlugin(_PLUGIN, service:getName() .. '-lastSelectedAlbumSet')

	if lastSelectedAlbumSet ~= nil and idHelper:findAlbumSetById(lastSelectedAlbumSet) then
		importTable.albumSet = lastSelectedAlbumSet
	elseif #albumSetsForPublisherService > 0 then
		local firstAlbum = TTGUtils.firstValue(albumSetsForPublisherService)
		importTable.albumSet = firstAlbum.id
	end

	local isBacklight = TTGUtils.isBacklight(publishSettings.apiURL)
	local isBacklight2 = TTGUtils.isBacklight2(collectionSettings, publishSettings)
	local isBacklight3 = TTGUtils.isBacklight3(collectionSettings, publishSettings)

	local importRow = f:row {
		bind_to_object = importTable,
		spacing = f:label_spacing(),

		f:static_text {
			title = "Copy Settings from Album Set",
			alignment = 'right',
			width = share 'title_width',
			enabled = #albumSetsForPublisherService > 0,
		},
		f:popup_menu {
			value = bind 'albumSet',
			items = bind {
				key = 'templates',
				bind_to_object = prefs,
				transform = function(v, t)
					local r = {}
					for k, v in pairs(albumSetsForPublisherService) do
						r[#r + 1] = { title = v.title, value = v.id }
					end
					return r
				end
			},
			enabled = #albumSetsForPublisherService > 0,
			fill_horizontal = 1
		},

		f:spacer {
			width = 5
		},
		f:push_button {
			title = "  Import  ",
			action = function()
				LrTasks.startAsyncTask(function()
					import_album_set(idHelper, info, importTable.albumSet)
				end)
			end,
			enabled = #albumSetsForPublisherService > 0,
		}
	}

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Title",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				enabled = true,
				value = bind 'title',
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width'
			},
			f:static_text {
				title = "Supports special characters.  Leave blank to use album set name for title.",
				alignment = 'left',
				size = 'small',
				width = 400,
			},
		})

	collectionSettings.oldslug = collectionSettings.slug
	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Slug",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				enabled = true,
				value = bind 'slug',
				validate = function(view, value)
					if value ~= nil then
						return true, TTGUtils.createSlugFromString(value, { preserve = publishSettings.preserveCapitalizationInAlbumSlugs })
					end
				end
			},
		})
	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width'
			},
			f:static_text {
				title = "Leave blank to generate slug from album set name.",
				alignment = 'left',
				size = 'small',
				width = 400,
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Description",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = 2,
				enabled = true,
				value = bind 'description'
			},
		})

	collectionSettings.hasAutoindexTemplates = yes -- #prefs.autoindexTemplates > 0

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Template",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'selectedTemplate',
				enabled = bind 'hasAutoindexTemplates',
				items = bind {
					key = 'autoindexTemplates',
					bind_to_object = prefs,
					transform = function(v, t)
						local r = {}
						r[#r + 1] = { title = 'No index', value = '' }
						for k, v in pairs(prefs.autoindexTemplates) do
							r[#r + 1] = { title = v.title, value = v.value }
						end
						return r
					end
				},
				fill_horizontal = 1
			},
		})

	if info.publishedCollection == nil or info.publishedCollection:getRemoteId() == nil then
		collectionSettings.thumbnails = {}
	-- For some reason, thumbnails is 'false' for Backlight2.
	elseif collectionSettings.thumbnails == nil or (type(collectionSettings.thumbnails) == 'table' and TTGUtils.tablelength(collectionSettings.thumbnails) == 0) then -- may have already been fetched with getTemplates
		LrTasks.startAsyncTask(function()
			collectionSettings.thumbnails = TTGApi.getThumbnails(info.publishService:getPublishSettings(), info.publishedCollection:getRemoteId())

			if collectionSettings.coverImage == nil and #collectionSettings.thumbnails > 0 then
				collectionSettings.coverImage = collectionSettings.thumbnails[1]
			end
		end)
	end

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Cover Image",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'coverImage',
				items = bind {
					key = 'thumbnails',
					bind_to_object = collectionSettings,
					transform = function(v, t)
						local r = {}
						r[#r + 1] = { title = 'Random', value = '' }
						r[#r + 1] = { title = 'Custom', value = 'custom_thumbnails/' }

						for k, thumbnail in pairs(collectionSettings.thumbnails) do
							--r[#r+1] = {title = v2.title, value = v2.value}
							r[#r + 1] = { title = '' .. thumbnail.title, value = '' .. thumbnail.value }
						end

						return r
					end
				},
				fill_horizontal = 1
			},
		})

	if isBacklight3 then
		if info.publishedCollection == nil or info.publishedCollection:getRemoteId() == nil then
			collectionSettings.photosForHeroImage = {}
			-- For some reason, photosForHeroImage is 'false' for Backlight2.
		else --if collectionSettings.photosForHeroImage == nil or (type(collectionSettings.photosForHeroImage) == 'table' and TTGUtils.tablelength(collectionSettings.photosForHeroImage) == 0) then -- may have already been fetched with getTemplates
			LrTasks.startAsyncTask(function()
				collectionSettings.photosForHeroImage = TTGApi.getPhotosForHeroImage(publishSettings, info.publishedCollection:getRemoteId())

				if collectionSettings.heroImage == nil and #collectionSettings.photosForHeroImage > 0 then
					collectionSettings.heroImage = collectionSettings.photosForHeroImage[1]
				end
			end)
		end

		table.insert(base_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Hero Image",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enableHeroImage'
				},
				f:popup_menu {
					value = bind 'heroImage',
					items = bind {
						key = 'photosForHeroImage',
						bind_to_object = collectionSettings,
						transform = function(v, t)
							local r = {}
							r[#r + 1] = { title = 'Random', value = '' }

							for k, photo in pairs(collectionSettings.photosForHeroImage) do
								r[#r + 1] = { title = '' .. photo.title, value = '' .. photo.value }
							end

							return r
						end
					},
					fill_horizontal = 1,
					enabled = bind 'enableHeroImage'
				},
			})
	end

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Album Order",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'ordering',
				items = TTGDialogs.albumSetOrdering,
				fill_horizontal = 1
			}
		})

	local tray1Title = ''
	local copyLines = 0
	local tray1PlacementOptions = nil

	if isBacklight then
		tray1Title = 'Pallet 1'
		copyLines = 3
		tray1PlacementOptions = TTGDialogs.tray1CopyOptions
	else
		tray1Title = 'Sidebar'
		copyLines = 5
		tray1PlacementOptions = TTGDialogs.sidebarCopyOptions
	end

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Main Copy (md)",
				alignment = 'right',
				width = share 'title_width',
			},

			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = copyLines,
				value = bind 'contentBlock',
			},
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Main Copy placement",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'pageCopyPlacement',
				items = TTGDialogs.pageCopyOptions,
				fill_horizontal = 1
			}
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = tray1Title.." Copy (md)",
				alignment = 'right',
				width = share 'title_width',
			},

			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = copyLines,
				value = bind 'sidebarCopy',
			},
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = tray1Title.." Copy placement",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'sidebarCopyPlacement',
				items = tray1PlacementOptions,
				fill_horizontal = 1
			}
		})

	if isBacklight then
		table.insert(page_copy_table,
			f:row {
				bind_to_object = collectionSettings,
				spacing = f:label_spacing(),

				f:static_text {
					title = "Pallet 2 Copy (md)",
					alignment = 'right',
					width = share 'title_width',
				},

				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					height_in_lines = copyLines,
					value = bind 'tray2Copy',
				},
			})

		table.insert(page_copy_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Pallet 2 Copy placement",
					alignment = 'right',
					width = share 'title_width',
				},
				f:popup_menu {
					value = bind 'tray2CopyPlacement',
					items = TTGDialogs.tray2CopyOptions,
					fill_horizontal = 1
				}
			})
	end

	if isBacklight2 then
	else
		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:spacer {
					width = share 'title_width',
				},
				f:checkbox {
					title = "Enable Password",
					value = bind 'enablePassword',
					enabled = bind 'allowEnablePassword'
				},
			})
	end

	if isBacklight2 then

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Visibility",
					alignment = 'right',
					width = share 'title_width',
				},
				f:popup_menu {
					value = bind 'visibility',
					items = TTGDialogs.visibilityOptions,
					fill_horizontal = 1,
					width = share 'input_width',
				}
			})

		local visibilityIsProtected = LrView.bind {
			key = 'visibility',
			transform = function (value, table)
				return value == 'protected'
			end
		}

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Access Code",
					alignment = 'right',
					width = share 'title_width',
					enabled = visibilityIsProtected,
				},
				f:password_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'password',
					enabled = visibilityIsProtected,
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})
	else
		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Username",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enablePassword',
				},
				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'username',
					enabled = bind 'enablePassword',
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Password",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enablePassword',
				},
				f:password_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'password',
					enabled = bind 'enablePassword',
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})
	end

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Display Breadcrumbs",
				value = bind 'showBreadcrumbs',
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Breadcrumb",
				alignment = 'right',
				width = share 'title_width',
				enabled = bind 'showBreadcrumbs',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				value = bind 'breadcrumb',
				enabled = bind 'showBreadcrumbs',
				validate = function(view, value)
					if value == nil or value == '' then
						return false, ""
					else
						value = TTGUtils.trim(value)
						if value == '' then
							return false, ""
						else
							return true, value
						end
					end
				end
			}
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Hide from Album Set",
				value = bind 'hideFromAutoindex',
			},
		})

	local the_column = {
		bind_to_object = collectionSettings, -- all controls bound to our table
		spacing = f:control_spacing(), -- default spacing for the child rows

		importRow,

		f:tab_view {
			bind_to_object = collectionSettings,
			f:tab_view_item {
				identifier = "tab1",
				title = "Base Settings",
				f:column(base_table)
			},
			f:tab_view_item {
				identifier = "tab2",
				title = "Page Content",
				f:column(page_copy_table)
			},
			f:tab_view_item {
				identifier = "tab3",
				title = "Features",
				f:column(feature_table)
			}
		}
	}

	return f:column(the_column)
end

function TTGDialogs.albumSettingsDialog(f, publishSettings, info)
	local collectionSettings = assert(info.collectionSettings)

	local service = assert(info.publishService)
	local idHelper = TTGGalleryIDHelper.new({ service = service })
	local albumsForPublisherService = idHelper:gatherAlbumsForPublisherService()

	local base_table = createSettingsTable(f, collectionSettings)
	local feature_table = createSettingsTable(f, collectionSettings)
	local page_copy_table = createSettingsTable(f, collectionSettings)
	local integration_table = createSettingsTable(f, collectionSettings)

	local cart_table = createSettingsTable(f, collectionSettings)
	local crg_table = createSettingsTable(f, collectionSettings)

	local importTable = LrFunctionContext.callWithContext('albumSettingsDialog', function(context)
		return LrBinding.makePropertyTable(context)
	end)

	local catalog = (import 'LrApplication').activeCatalog()


	local lastSelectedAlbum = catalog:getPropertyForPlugin(_PLUGIN, service:getName() .. '-lastSelectedAlbum')

	if lastSelectedAlbum ~= nil and idHelper:findAlbumById(lastSelectedAlbum) then
		importTable.album = lastSelectedAlbum
	elseif #albumsForPublisherService > 0 then
		local firstAlbum = TTGUtils.firstValue(albumsForPublisherService)
		importTable.album = firstAlbum.id
	end

	local isBacklight = TTGUtils.isBacklight(publishSettings.apiURL)
	local isBacklight2 = TTGUtils.isBacklight2(collectionSettings, publishSettings)
	local isBacklight3 = TTGUtils.isBacklight3(collectionSettings, publishSettings)

	local importRow = f:row {
		bind_to_object = importTable,
		spacing = f:label_spacing(),

		f:static_text {
			title = "Copy Settings from Album",
			alignment = 'right',
			width = share 'title_width',
			enabled = #albumsForPublisherService > 0,
		},
		f:popup_menu {
			value = bind 'album',
			items = bind {
				key = 'templates',
				bind_to_object = prefs,
				transform = function(v, t)
					local r = {}
					for k, v in pairs(albumsForPublisherService) do
						if v.title ~= nil and v.id ~= nil then
							r[#r + 1] = { title = v.title, value = v.id }
						end
					end
					return r
				end
			},
			enabled = #albumsForPublisherService > 0,
			fill_horizontal = 1
		},

		f:spacer {
			width = 5
		},
		f:push_button {
			title = "  Import  ",
			action = function()
				LrTasks.startAsyncTask(function()
					import_album(idHelper, info, importTable.album)
				end)
			end,
			enabled = #albumsForPublisherService > 0
		}
	}

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Album Title",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				enabled = true,
				value = bind 'title',
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			-- spacing = f:label_spacing(),
			f:spacer {
				width = share 'title_width'
			},
			f:static_text {
				title = "Supports special characters.  Leave blank to use album name for title.",
				alignment = 'left',
				size = 'small',
				width = 400,
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Album Description",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = 2,
				enabled = true,
				value = bind 'description'
			},
		})

	collectionSettings.oldslug = collectionSettings.slug
	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Slug",
				alignment = 'right',
				width = share 'title_width',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				enabled = true,
				value = bind 'slug',
				validate = function(view, value)
					if value ~= nil then
						return true, TTGUtils.createSlugFromString(value, { preserve = publishSettings.preserveCapitalizationInAlbumSlugs })
					end
				end
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			-- spacing = f:label_spacing(),
			f:spacer {
				width = share 'title_width'
			},
			f:static_text {
				title = "Leave blank to generate slug from album name.",
				alignment = 'left',
				size = 'small',
				width = 400,
			},
		})

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Template",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'selectedTemplate',
				items = bind {
					key = 'templates',
					bind_to_object = prefs,
					transform = function(v, t)
						local r = {}
						for k, v in pairs(prefs.templates) do
							r[#r + 1] = { title = v.title, value = v.value }
						end
						return r
					end
				},
				fill_horizontal = 1
			},
		})


	if info.publishedCollection == nil or info.publishedCollection:getRemoteId() == nil then
		collectionSettings.thumbnails = {}
	-- For some reason, thumbnails is 'false' for Backlight2.
	elseif collectionSettings.thumbnails == nil or (type(collectionSettings.thumbnails) == 'table' and TTGUtils.tablelength(collectionSettings.thumbnails) == 0) then -- may have already been fetched with getTemplates
		LrTasks.startAsyncTask(function()
			collectionSettings.thumbnails = TTGApi.getThumbnails(publishSettings, info.publishedCollection:getRemoteId())

			if collectionSettings.coverImage == nil and #collectionSettings.thumbnails > 0 then
				collectionSettings.coverImage = collectionSettings.thumbnails[1]
			end
		end)
	end

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Cover Image",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'coverImage',
				items = bind {
					key = 'thumbnails',
					bind_to_object = collectionSettings,
					transform = function(v, t)
						local r = {}
						r[#r + 1] = { title = 'Random', value = '' }
						r[#r + 1] = { title = 'Custom', value = 'custom_thumbnails/' }

						for k, thumbnail in pairs(collectionSettings.thumbnails) do
							r[#r + 1] = { title = '' .. thumbnail.title, value = '' .. thumbnail.value }
						end

						return r
					end
				},
				fill_horizontal = 1
			},
		})

	if isBacklight3 then
		if info.publishedCollection == nil or info.publishedCollection:getRemoteId() == nil then
			collectionSettings.photosForHeroImage = {}
			-- For some reason, photosForHeroImage is 'false' for Backlight2.
		else --if collectionSettings.photosForHeroImage == nil or (type(collectionSettings.photosForHeroImage) == 'table' and TTGUtils.tablelength(collectionSettings.photosForHeroImage) == 0) then -- may have already been fetched with getTemplates
			LrTasks.startAsyncTask(function()
				collectionSettings.photosForHeroImage = TTGApi.getPhotosForHeroImage(publishSettings, info.publishedCollection:getRemoteId())

				if collectionSettings.heroImage == nil and #collectionSettings.photosForHeroImage > 0 then
					collectionSettings.heroImage = collectionSettings.photosForHeroImage[1]
				end
			end)
		end

		table.insert(base_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Hero Image",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enableHeroImage'
				},
				f:popup_menu {
					value = bind 'heroImage',
					items = bind {
						key = 'photosForHeroImage',
						bind_to_object = collectionSettings,
						transform = function(v, t)
							local r = {}
							r[#r + 1] = { title = 'Random', value = '' }

							for k, photo in pairs(collectionSettings.photosForHeroImage) do
								r[#r + 1] = { title = '' .. photo.title, value = '' .. photo.value }
							end

							return r
						end
					},
					fill_horizontal = 1,
					enabled = bind 'enableHeroImage'
				},
			})
	end

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Items per page",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'itemsPerPage',
				items = TTGDialogs.itemsPerPage,
				fill_horizontal = 1
			}
		})

	--[[
	
			f:slider {
					value = bind 'itemsPerPage',
					items = TTGDialogs.itemsPerPage,
					min = 1,
					max = 250, 
					integral = true,
					fill_horizontal = 0
				},
				f:edit_field { 
					fill_horizonal = 1,
					width_in_chars = 6, 
					value = bind 'itemsPerPage',
					validate = function(view,value)
						if value ~= nil then
							local intVal = math.floor(value)
							logger:debug('intVal: '..intVal)
							
							local returnVal = 20
							if intVal > 0 and intVal < 251 then
								returnVal = intVal
							end
							
							return true, value --""..intVal 
							--return true--,TTGUtils.addTrailingSlashIfNeeded(value)
						end
					end
				}
	
	]] --

	table.insert(base_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Photo Order",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'ordering',
				items = TTGDialogs.ordering,
				fill_horizontal = 1
			}
		})

	local tray1Title = ''
	local copyLines = 0
	local tray1PlacementOptions = nil

	if isBacklight then
		tray1Title = 'Pallet 1'
		copyLines = 3
		tray1PlacementOptions = TTGDialogs.tray1CopyOptions
	else
		tray1Title = 'Sidebar'
		copyLines = 5
		tray1PlacementOptions = TTGDialogs.sidebarCopyOptions
	end

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = "Main Copy (md)",
				alignment = 'right',
				width = share 'title_width',
			},

			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = copyLines,
				value = bind 'contentBlock',
			},
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Main Copy placement",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'pageCopyPlacement',
				items = TTGDialogs.pageCopyOptions,
				fill_horizontal = 1
			}
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			spacing = f:label_spacing(),

			f:static_text {
				title = tray1Title.." Copy (md)",
				alignment = 'right',
				width = share 'title_width',
			},

			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				height_in_lines = copyLines,
				value = bind 'sidebarCopy',
			},
		})

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = tray1Title.." Copy placement",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'sidebarCopyPlacement',
				items = tray1PlacementOptions,
				fill_horizontal = 1
			}
		})

	if isBacklight then
		table.insert(page_copy_table,
			f:row {
				bind_to_object = collectionSettings,
				spacing = f:label_spacing(),

				f:static_text {
					title = "Pallet 2 Copy (md)",
					alignment = 'right',
					width = share 'title_width',
				},

				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					height_in_lines = copyLines,
					value = bind 'tray2Copy',
				},
			})

		table.insert(page_copy_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Pallet 2 Copy placement",
					alignment = 'right',
					width = share 'title_width',
				},
				f:popup_menu {
					value = bind 'tray2CopyPlacement',
					items = TTGDialogs.tray2CopyOptions,
					fill_horizontal = 1
				}
			})
	end

	table.insert(page_copy_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Video ID",
				alignment = 'right',
				width = share 'title_width',
				enabled = bind 'enableVideo',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				value = bind 'videoID',
				enabled = bind 'enableVideo',
				validate = function(view, value)
					if value == nil or value == '' then
						return false, ""
					else
						value = TTGUtils.trim(value)
						return true, value
					end
				end
			}
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Enable Maps",
				value = bind 'enableMaps',
				enabled = bind 'allowEnableMaps'
			},
		})

	if isBacklight2 then
	else
		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:spacer {
					width = share 'title_width',
				},
				f:checkbox {
					title = "Enable Password",
					value = bind 'enablePassword',
					enabled = bind 'allowEnablePassword'
				},
			})
	end

	if isBacklight2 then

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Visibility",
					alignment = 'right',
					width = share 'title_width',
				},
				f:popup_menu {
					value = bind 'visibility',
					items = TTGDialogs.visibilityOptions,
					fill_horizontal = 1,
					width = share 'input_width',
				}
			})

		local visibilityIsProtected = LrView.bind {
			key = 'visibility',
			transform = function (value, table)
				return value == 'protected'
			end
		}

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Access Code",
					alignment = 'right',
					width = share 'title_width',
					enabled = visibilityIsProtected,
				},
				f:password_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'password',
					enabled = visibilityIsProtected,
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})
	else
		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Username",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enablePassword',
				},
				f:edit_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'username',
					enabled = bind 'enablePassword',
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})

		table.insert(feature_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Password",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'enablePassword',
				},
				f:password_field {
					fill_horizonal = 1,
					width_in_chars = 35,
					value = bind 'password',
					enabled = bind 'enablePassword',
					validate = function(view, value)
						if value == nil or value == '' then
							return false, "guest"
						else
							value = TTGUtils.trim(value)
							if value == '' then
								return false, "guest"
							else
								return true, value
							end
						end
					end
				}
			})
	end

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Display Breadcrumbs",
				value = bind 'showBreadcrumbs',
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Breadcrumb",
				alignment = 'right',
				width = share 'title_width',
				enabled = bind 'showBreadcrumbs',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				value = bind 'breadcrumb',
				enabled = bind 'showBreadcrumbs',
				validate = function(view, value)
					if value == nil or value == '' then
						return false, ""
					else
						value = TTGUtils.trim(value)
						if value == '' then
							return false, ""
						else
							return true, value
						end
					end
				end
			}
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width'
			},
			f:static_text {
				title = "Leave blank to use album title for breadcrumb.",
				enabled = bind 'showBreadcrumbs',
				alignment = 'left',
				size = 'small',
				width = 400,
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Watermark to apply",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
				value = bind 'watermark',
				items = TTGUtils.getWatermarkDropdown(),
				fill_horizontal = 1
			}
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Exclude watermarks in thumbnails",
				value = bind 'excludeWatermarksInThumbnails',
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Exclude watermarks in photos-for-purchase",
				value = bind 'excludeWatermarksInPhotosForPurchase',
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Hide from Album Set",
				value = bind 'hideFromAutoindex',
			},
		})

	table.insert(feature_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Hide from Search",
				value = bind 'hideFromSearch',
			},
		})


	--[[
	table.insert(cart_table, 	
		f:row {
			spacing = f:label_spacing(),
			f:static_text {
				title = "Cart",
				alignment = 'left',
				-- width = share 'title_width',
			},
		}
	) ]] --

	table.insert(cart_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Enable Cart",
				value = bind 'enableCart',
				enabled = bind 'allowEnableCart'
			},
		})

	table.insert(cart_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Cart Pricing Profile",
				alignment = 'right',
				width = share 'title_width',
				enabled = bind 'enableCart',
			},
			f:edit_field {
				fill_horizonal = 1,
				width_in_chars = 35,
				width = share 'input_width',
				value = bind 'pricing',
				enabled = bind 'enableCart',
				validate = function(view, value)
					if value == nil or value == '' then
						return false, "default"
					else
						value = TTGUtils.trim(value)
						if value == '' then
							return false, "default"
						else
							return true, value
						end
					end
				end
			}
		})

	if isBacklight2 then
	else
		table.insert(crg_table,
			f:row {
				bind_to_object = collectionSettings,
				f:static_text {
					title = "Client Management",
					alignment = 'right',
					width = share 'title_width',
					enabled = bind 'allowEnableCRG' --'enableCRG'
				},
				f:popup_menu {
					value = bind 'crgClientMode',
					items = TTGDialogs.crgClientModes,
					fill_horizontal = 1,
					width = share 'input_width',
					enabled = bind 'allowEnableCRG' --'enableCRG'
				}
			})
	end


	table.insert(crg_table,
		f:row {
			bind_to_object = collectionSettings,
			f:spacer {
				width = share 'title_width',
			},
			f:checkbox {
				title = "Enable Feedback",
				value = bind 'enableCRG',
				enabled = bind 'allowEnableCRG'
			},
		})

	table.insert(crg_table,
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Standalone Feedback Profile",
				alignment = 'right',
				width = share 'title_width',
				enabled = bind 'enableCRG'
			},
			f:popup_menu {
				value = bind 'crgFeedbackProfile',
				items = bind {
					key = 'crgFeedbackProfiles',
					bind_to_object = prefs,
					transform = function(v, t)
						local r = {}
						for k, v in pairs(prefs.crgFeedbackProfiles) do
							r[#r + 1] = { title = v.title, value = v.value }
						end
						return r
					end
				},
				fill_horizontal = 1,
				width = share 'input_width',
				enabled = bind 'enableCRG'
			}
		})

--	enabled = bind 'enableCRG'


	table.insert(integration_table,
		f:row {
			spacing = f:label_spacing(),
			f:static_text {
				title = "Cart",
				alignment = 'left',
				enabled = bind 'allowEnableCart',
				visible = no
			},
		})

	table.insert(integration_table,
		f:row {
			spacing = f:label_spacing(),
			f:group_box {
				title = "Cart",
				show_title = false,
				width = share 'cart_box_width',
				f:column(cart_table)
			}
		})

	table.insert(integration_table,
		f:row {
			spacing = f:label_spacing(),
			f:static_text {
				title = "Client Response",
				alignment = 'left',
				enabled = bind 'allowEnableCRG',
				visible = no
			},
		})

	table.insert(integration_table,
		f:row {
			spacing = f:label_spacing(),
			f:group_box {
				title = "Client Response",
				show_title = false,
				f:column(crg_table)
			}
		})

	local the_column = {
		bind_to_object = collectionSettings, -- all controls bound to our table
		spacing = f:control_spacing(), -- default spacing for the child rows

		importRow,

		f:tab_view {
			bind_to_object = collectionSettings,
			f:tab_view_item {
				identifier = "tab1",
				title = "Base Settings",
				f:column(base_table)
			},
			f:tab_view_item {
				identifier = "tab2",
				title = "Page Content",
				f:column(page_copy_table)
			},
			f:tab_view_item {
				identifier = "tab3",
				title = "Features",
				f:column(feature_table)
			},
			f:tab_view_item {
				identifier = "tab4",
				title = "Integration",
				enabled = yes,
				f:column(integration_table)
			}
		}
	}

	return f:column(the_column)
end

function TTGDialogs.showDuplicateGalleryDialog()
	return LrFunctionContext.callWithContext('duplicate gallery dialog', function(context)

		local f = LrView.osFactory()
		local properties = LrBinding.makePropertyTable(context)

		properties.selectedButton = 'update'

		local contents = f:column {
			bind_to_object = properties,
			spacing = f:control_spacing(),
			fill = 1,

			f:row {
				spacing = f:label_spacing(),
				f:static_text {
					title = "There is already an existing gallery with the same slug.  \nWhat would you like to do?",
					alignment = 'left',
					-- width = share 'title_width',
				},
			},
			f:column {
				alignment = 'center',
				spacing = f:label_spacing(),
				width = share 'title_width',

				f:row {
					f:spacer {
						height = 15
					},
				},

				f:row {
					spacing = f:label_spacing(),
					f:spacer {
						width = 80
					},
					f:radio_button {
						title = "Update existing gallery",
						width = share 'title_width',
						checked_value = "update",
						value = LrView.bind("selectedButton"),
					},
				},
				f:row {
					f:spacer {
						height = 3
					},
				},
				f:row {
					spacing = f:label_spacing(),
					f:spacer {
						width = 80
					},
					f:radio_button {
						title = "Create new gallery",
						width = share 'title_width',
						checked_value = "new",
						value = LrView.bind("selectedButton"),
					},
				},
				f:row {
					f:spacer {
						height = 7
					},
				},
			}
		}

		while true do
			local result = LrDialogs.presentModalDialog {
				title = "Duplicate gallery found",
				contents = contents,
			}

			if result == 'ok' then
				if properties.selectedButton ~= nil then
					return properties.selectedButton
				end
			else
				LrErrors.throwCanceled()
			end
		end
	end)
end

function TTGDialogs.showDuplicateGalleryProceedDialog()
	return LrFunctionContext.callWithContext('duplicate gallery proceed dialog', function(context)

		local f = LrView.osFactory()
		local properties = LrBinding.makePropertyTable(context)

		properties.selectedButton = 'update'

		local contents = f:column {
			bind_to_object = properties,
			spacing = f:control_spacing(),
			fill = 1,

			f:row {
				spacing = f:label_spacing(),
				f:static_text {
					title = "There is already an existing gallery with the same slug.  \nIf you proceed, this gallery will have a number automatically appended to the slug.  \nWould you like to proceed?",
					alignment = 'left',
					-- width = share 'title_width',
				},
			},
		}

		while true do
			local result = LrDialogs.presentModalDialog {
				title = "Duplicate gallery found",
				contents = contents,
			}

			if result == 'ok' then
				if properties.selectedButton ~= nil then
					return 'proceed'
				end
			else
				LrErrors.throwCanceled()
			end
		end
	end)
end

return TTGDialogs;
