require 'TTGMigrator'

local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrFunctionContext = import 'LrFunctionContext' local LrBinding = import 'LrBinding'
local LrTasks = import 'LrTasks'
local LrView = import 'LrView'

local bind = LrView.bind
local share = LrView.share

local LrLogger = import 'LrLogger'
local logger = LrLogger( 'ttg' )
logger:enable( "logfile" )

TTGMigratorExportItem = {}

function TTGMigratorExportItem.showMigrationDialogue(ce3_services, ce4_services)
	return LrFunctionContext.callWithContext( 'migrate dialog', function( context )
	local key
	local url

	local f = LrView.osFactory()
	local properties = LrBinding.makePropertyTable( context )
	
	if #ce3_services > 0 then
		properties.ce3Service = ce3_services[1].value
	end
	
	if #ce4_services > 0 then
		properties.ce4Service = ce4_services[1].value
	end

	local contents = f:column {
		bind_to_object = properties,
		spacing = f:control_spacing(),
		fill = 1,
      
		f:row {
			bind_to_object = collectionSettings,
			f:static_text {
				title = "Copy the Album Sets and Albums from a CE2 or CE3 Publisher\nto an empty CE4 Publisher.",
				alignment = 'left'
			},
		},
		
		f:row {
			f:spacer {
				height = 10
			},
		},

		f:row {
			bind_to_object = properties,
			f:static_text {
				title = "Source",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
					value = bind 'ce3Service',
					items = ce3_services,
					validate = function(view,value)
						end,
					fill_horizontal = 1
				},
			},
			
		f:row {
			bind_to_object = properties,
			f:static_text {
				title = "Destination",
				alignment = 'right',
				width = share 'title_width',
			},
			f:popup_menu {
					value = bind 'ce4Service',
					items = ce4_services,
					validate = function(view,value)
						end,
					fill_horizontal = 1
			},
		},
    }
    
	
	while true do
	local result = LrDialogs.presentModalDialog {
		title = "Migrate Publisher", 
		contents = contents,
	}
	
	if result == 'ok' then
		return properties.ce3Service, properties.ce4Service
	else
		LrErrors.throwCanceled()	
	end
      end
  end)
end



function TTGMigratorExportItem.migrate_service()

	LrTasks.startAsyncTask( function()
		local catalog = (import 'LrApplication').activeCatalog()
		
		local ce3_services = TTGMigrator.getServicesForPublisherId("com.ttg.ce2.publisher", "TTG CE3 Publisher")
		local ce4_services = TTGMigrator.getServicesForPublisherId("com.ttg.ce4.publisher", "TTG CE4 Publisher")

		if #ce3_services ==  0 then
			LrDialogs.message("Error", "There are no instances of CE3 Publisher to migrate from.")
			logger:debug('Migrator::There are no instances of CE3 Publisher to migrate from.')
			return
		end

		if #ce4_services == 0 then
			LrDialogs.message("Error", "There are no instances of CE4 Publisher to migrate to.  Create a new instance of CE4 Publisher and try again.")
			logger:debug('Migrator::There are no instances of CE4 Publisher to migrate to.  Create a new instance of CE4 Publisher and try again.')
			return
		end

		local ce3ServiceIndex,ce4ServiceIndex = TTGMigratorExportItem.showMigrationDialogue(ce3_services, ce4_services)
		--handle cancel
		if not ce3ServiceIndex or not ce4ServiceIndex then 
			return
		end
		
		local source = ce3_services[ce3ServiceIndex].service-- TTGMigrator.getServiceForPublisherIdAndName("com.ttg.ce2.publisher", ce3ServiceName)
		local destination = ce4_services[ce4ServiceIndex].service -- TTGMigrator.getServiceForPublisherIdAndName("com.ttg.ce4.publisher", ce4ServiceName)

		local sourceSettings = source:getPublishSettings()
		local destinationSettings = destination:getPublishSettings()
		
		local sourceBase =  sourceSettings.apiURL:gsub("publisher/api/", "")
		local destinationBase =  destinationSettings.apiURL:gsub("ttg%-be/publisher/", "")

		logger:debug('Migrator::source apiURL: '..sourceSettings.apiURL)
		logger:debug('Migrator::source galleryName: '..sourceSettings.galleryName)
		logger:debug('Migrator::destination apiURL: '..destinationSettings.apiURL)
		logger:debug('Migrator::destination galleryName: '..destinationSettings.galleryName)
		logger:debug('Migrator::sourceBase: '..sourceBase)
		logger:debug('Migrator::destinationBase: '..destinationBase)

		if sourceBase ~= destinationBase then
			LrDialogs.message("The settings do not match", "The API URL must match between Source and Destination.\n\nUpdate these settings in the destination CE4 Publisher instance and try again.\n\n"..sourceBase.."\n\n"..destinationBase)
			logger:debug("Migrator::The settings do not match", "The API URL must match between Source and Destination.\n\nUpdate these settings in the destination CE4 Publisher instance and try again.")
			return
		end

		if sourceSettings.galleryName ~=  destinationSettings.galleryName then
			LrDialogs.message("The settings do not match", "The top-level galleries directory settings must match between the Source and Destination.\n\nUpdate these settings in the destination CE4 Publisher instance and try again.")
			logger:debug("Migrator::The settings do not match", "The top-level galleries directory settings must match between the Source and Destination.\n\nUpdate these settings in the destination CE4 Publisher instance and try again.")
			return
		end
		
		if source then
			logger:debug("Migrator::source found: "..source:getName())
		end
		
		if destination then
			logger:debug("Migrator::destination found: "..destination:getName())
		end
		
		local destinationChildAlbumSets = destination:getChildCollectionSets();
		local destinationChildAlbums = destination:getChildCollections();
		
		if #destinationChildAlbumSets > 0 or #destinationChildAlbums > 1 or #destinationChildAlbums == 1 and destinationChildAlbums[1]:getName() ~= 'Default Album' then
			LrDialogs.message("Destination not empty", "The destination Publisher instance is not empty. Migration can only be made to an empty CE4 Publisher instance.")
		else
			--local migrated_album_ids = {"43749","65207","95411","51636","57473","1190"}

			local idHelper = TTGGalleryIDHelper.new({service = source})
			local idsOnService = idHelper:delimitedIDListForService()
			
			logger:debug("Migrator::ids on service: "..idsOnService)
			local migrated_album_ids = TTGApi.migrateGalleries(sourceSettings.galleryName, idsOnService, destinationSettings)
			logger:debug("Migrator::migrator_album_ids: "..TTGUtils.inspect(migrated_album_ids))
			
			local ttgMigrator = TTGMigrator.new({source = source, destination = destination, migrated_album_ids = migrated_album_ids})
			ttgMigrator:copyPublishService()
			local migrated = ttgMigrator:getMigrated()
			local omitted = ttgMigrator:getOmitted()
			
			local migratedMessage = nil
			local omittedMessage = nil
			local message = '';
			
			
			if #migrated > 0 then
				message = "Migration was successfully completed from "..source:getName().." to "..destination:getName()
				
				if #migrated < 15 then
					migratedMessage = "The following "..#migrated.." albums and album sets were migrated: \n"
					for i=1,#migrated,1 do
						migratedMessage = migratedMessage .. "\n" .. migrated[i]:getName()
					end
				else
					migratedMessage = "A total of "..#migrated.." albums and album sets were migrated"
				end
			else
				message = "None of the albums or album sets from the " .. source:getName() .. " publish service were found on the server"
			end
			
			if #omitted > 0 then
				omittedMessage = "The following albums and album sets were not found on the server and have not been migrated: \n"
				for i=1,#omitted,1 do
					omittedMessage = omittedMessage .. "\n" .. omitted[i]:getName()
				end
			end
			
			if migratedMessage then
				message = message .. "\n\n"..migratedMessage
			end
			
			if omittedMessage then
				message = message .. "\n\n"..omittedMessage
			end
			
			local title = "Success"
			
			if #migrated == 0 then
				title = "There was a problem"
			end
			
			LrDialogs.message(title, message)
			logger:debug("Migrator::Migration was successfully completed from "..source:getName().." to "..destination:getName())
		end
	end)
end

TTGMigratorExportItem.migrate_service()
	