-- TTG imports
require 'TTGApi'
require 'TTGGalleryIDHelper'
require 'TTGDialogs'
require 'TTGUtils'
require 'MetadataHandler'

-- Lightroom SDK
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrExportSession = import 'LrExportSession'
local LrFileUtils = import 'LrFileUtils'
local LrHttp = import 'LrHttp'
local LrPathUtils = import 'LrPathUtils'
local LrTasks = import 'LrTasks'
local LrView = import 'LrView'

-- Common shortcuts
local bind = LrView.bind
local share = LrView.share
local prefs = import 'LrPrefs'.prefsForPlugin()

local LrLogger = import 'LrLogger'
local logger = LrLogger("ttg")
logger:enable( "logfile" )

-------------------------------------------------------------------------------------------------------------------------------------
--
--   Settings
--
-------------------------------------------------------------------------------------------------------------------------------------

local publishServiceProvider = {}

--publishServiceProvider.hideSections={'exportLocation','video','imageSettings','fileNaming','fileSettings','outputSharpening','metadata'}
--publishServiceProvider.hideSections={'exportLocation','video','imageSettings','fileNaming','fileSettings', 'outputSharpening'}
publishServiceProvider.hideSections={'exportLocation','video','imageSettings','fileSettings', 'outputSharpening'}

-- LOOK to add:
-------------------------
-- metaData --- allowed.  Look to use all/copyright-only as set by template.  Also, set to copyright-only for non-photos.
-- outputSharpening --- need to review handling of sharpening as set in template
-- fileNaming --- issues with spaces, and need to delete files of previous name if scheme changed
-- fileSettings --- Limit file size to xK not to be applied to all rendition types

publishServiceProvider.allowFileFormats = { 'JPEG' }
publishServiceProvider.supportsIncrementalPublish = 'only'
publishServiceProvider.small_icon = 'supporting_files/ttg-icon.png'
publishServiceProvider.titleForPublishedCollection = 'Album'
publishServiceProvider.titleForPublishedCollectionSet = 'Album Set'
publishServiceProvider.titleForPublishedSmartCollection = "Smart Album"
publishServiceProvider.titleForGoToPublishedCollection = 'Go to Published Album'
publishServiceProvider.supportsCustomSortOrder = true
publishServiceProvider.disableRenamePublishedCollection = false

local writeTimeout = { timeout=60, asynchronous=true }

publishServiceProvider.exportPresetFields = {
	{key="templates", default={}},
	{key="autoindexTemplates", default={}},
	{key="apiURL", default=nil},
	{key="apiKey", default=nil},
	{key="apiSalt", 'xd*8T_-g`ZJA48&;6!97j9%|;iD*k]C-R:52w8(~OICERoZkDEf&*_vuz-OHFI)w'},
	{key="httpUsername", ''},
	{key="httpPassword", ''},
	{key="galleryName", default='galleries'},
	{key="openAlbumAfterExport", default=false},
	{key="preserveCapitalizationInAlbumSlugs", default=false},
	{key="pushMetadataOnly", default=false},
	{key="publishThumbnailsOnly", default=false},
	{key="delayAPIRequests", default=false},
	{key="delayAPIRequestsSeconds", default=5},
	{key="showCartOptions", default=false},
	{key="showPasswordProtectionOptions", default=false},
}

-------------------------------------------------------------------------------------------------------------------------------------
--
--   Custom functions
--
-------------------------------------------------------------------------------------------------------------------------------------

function createBaseExportSettings(propertyTable, requiredRendition)

	local exportSettings = assert( propertyTable )

	exportSettings.LR_outputSharpeningOn = true

	if requiredRendition.sharpening ~= nil and requiredRendition.sharpening == 'off' then
		exportSettings.LR_outputSharpeningOn = false
	else
		exportSettings.LR_outputSharpeningOn = true

		if requiredRendition.sharpeningMedia ~= nil and (requiredRendition.sharpeningMedia == 'screen' or requiredRendition.sharpeningMedia == 'matte' or requiredRendition.sharpeningMedia == 'glossy') then
			exportSettings.LR_outputSharpeningMedia = requiredRendition.sharpeningMedia
		else
			exportSettings.LR_outputSharpeningMedia = 'screen'
		end

		if requiredRendition.sharpeningLevel ~= nil and (requiredRendition.sharpeningLevel == 1 or requiredRendition.sharpeningLevel == 2 or requiredRendition.sharpeningLevel == 3) then
			exportSettings.LR_outputSharpeningLevel = requiredRendition.sharpeningLevel
		else
			exportSettings.LR_outputSharpeningLevel = 2
		end
	end

	logger:debug('sharpening path: '..requiredRendition.renditionPath)
	if exportSettings.LR_outputSharpeningOn then
		logger:debug('sharpening is ON')
		logger:debug('sharpening media: '..exportSettings.LR_outputSharpeningMedia)
		logger:debug('sharpening level: '..exportSettings.LR_outputSharpeningLevel)
	else
		logger:debug('sharpening is OFF')
	end

	if requiredRendition.quality ~= nil then
		exportSettings.LR_jpeg_quality = 0 + requiredRendition.quality
	else
		exportSettings.LR_jpeg_quality = 0.76
	end

	if requiredRendition.colorSpace ~= nil and (requiredRendition.colorSpace == 'sRGB' or requiredRendition.colorSpace == 'AdobeRGB' or requiredRendition.colorSpace == 'ProPhotoRGB') then
		exportSettings.LR_export_colorSpace = requiredRendition.colorSpace
	else
		exportSettings.LR_export_colorSpace = 'sRGB'
	end

	logger:debug('colorSpace: '..exportSettings.LR_export_colorSpace..' for '..requiredRendition.renditionPath)

	--[[if requiredRendition.colorSpace ~= nil and requiredRendition.colorSpace == 'AdobeRGB' then
		exportSettings.LR_export_colorSpace = 'AdobeRGB'
		logger:debug('set colorSpace to AdobeRGB for '..requiredRendition.renditionPath)
	else
		exportSettings.LR_export_colorSpace = 'sRGB'
		logger:debug('set colorSpace to sRGB for '..requiredRendition.renditionPath)
	end]]
	
	exportSettings.LR_export_destinationType = 'tempFolder'

	-- it appears that LR can't figure this out by itself when creating a new LrExportSession
	exportSettings.LR_export_destinationPathPrefix = LrPathUtils.getStandardFilePath('temp')
	exportSettings.LR_export_useSubfolder = false
	exportSettings.LR_collisionHandling = 'overwrite'
	--exportSettings.LR_collisionHandling = 'ask'

	-- Found this by following advice here here http://forums.adobe.com/message/4610626?tstart=0#4610626
	exportSettings.LR_size_doConstrain = true
	exportSettings.LR_size_doNotEnlarge = true

	-- Backlight has traditionally sent these back as strings.  This caused the size settings to be ignored in LR 11.4.
	requiredRendition.maxWidth = tonumber(requiredRendition.maxWidth)
	requiredRendition.maxHeight = tonumber(requiredRendition.maxHeight)
	requiredRendition.constraintLength = tonumber(requiredRendition.constraintLength)

	if requiredRendition.resizeType == 'maxHeight' then
		exportSettings.LR_size_maxHeight = requiredRendition.constraintLength
	elseif requiredRendition.resizeType == 'wh' then
		exportSettings.LR_size_resizeType = requiredRendition.resizeType
		exportSettings.LR_size_maxWidth= requiredRendition.maxWidth
		exportSettings.LR_size_maxHeight = requiredRendition.maxHeight
	else
		exportSettings.LR_size_resizeType = requiredRendition.resizeType
		exportSettings.LR_size_maxHeight = requiredRendition.constraintLength
	end
	
	if requiredRendition.renditionPath == 'thumbnails' or requiredRendition.renditionPath == 'thumbnails-for-mobile' then
		exportSettings.LR_embeddedMetadataOption = "copyrightOnly"
		exportSettings.LR_minimizeEmbeddedMetadata = true
	else
		exportSettings.LR_embeddedMetadataOption = propertyTable.LR_embeddedMetadataOption
		exportSettings.LR_minimizeEmbeddedMetadata = propertyTable.LR_minimizeEmbeddedMetadata
	end
	
	return exportSettings
end

function calculateDimensions(photo, requiredRendition)

	local isCropped = photo:getRawMetadata('isCropped');
	local dimensions = {}

	if isCropped then
		dimensions = photo:getRawMetadata('croppedDimensions')
	else
		dimensions = photo:getRawMetadata('dimensions')
	end

	local ratio = dimensions.width / dimensions.height
	local width = 0
	local height = 0

	if requiredRendition.resizeType == 'longEdge' then
		if ratio > 1 then
			width = requiredRendition.constraintLength
			height = TTGUtils.round(requiredRendition.constraintLength / ratio)
		else
			width = TTGUtils.round(requiredRendition.constraintLength * ratio)
			height = requiredRendition.constraintLength
		end
	elseif requiredRendition.resizeType == 'maxWidth' then
		height = TTGUtils.round(requiredRendition.constraintLength / ratio)
		width = requiredRendition.constraintLength
	elseif requiredRendition.resizeType == 'maxHeight' then
		width = TTGUtils.round(requiredRendition.constraintLength * ratio)
		height = requiredRendition.constraintLength
	elseif requiredRendition.resizeType == 'wh' then
		if ratio > 1 then
			height = TTGUtils.round(requiredRendition.maxWidth / ratio)
			
			if height > 0 + requiredRendition.maxHeight then
				height = requiredRendition.maxHeight
				width = TTGUtils.round(height * ratio)
			else
				width = requiredRendition.maxWidth
			end
		else
			width = TTGUtils.round(requiredRendition.maxHeight * ratio)
			
			if width > 0 + requiredRendition.maxWidth then
				width = requiredRendition.maxWidth
				height = TTGUtils.round(width / ratio)
			else
				height = requiredRendition.maxHeight
			end
		end
	else	
		if ratio > 1 then
			width = TTGUtils.round(requiredRendition.constraintLength * ratio)
			height = requiredRendition.constraintLength
		else
			width = requiredRendition.constraintLength
			height = TTGUtils.round(requiredRendition.constraintLength / ratio)
		end
	end

	if tonumber(width) > dimensions.width or tonumber(height) > dimensions.height then
		return dimensions.width,dimensions.height
	end
	
	return width,height	
end

function hasPublishedRendition( photoId, renditionName, photos )

	if photos == nil or photos[photoId] == nil then
		return false
	end
	
	local photo = photos[photoId]
	
	if photo['renditions'] == nil then
		return false
	end
	
	if photo['renditions'][renditionName] == nil then
		return false
	end
	
	return true

end

function getParentId(info)
	if info.parents == nil or #info.parents == 0 then
		return ''
	end

	local parentId = ''

	for k,v in pairs(info.parents) do
		parentId = v.remoteCollectionId
	end

	return parentId
end

function setGenericDefaults(publishSettings, collectionSettings)
	if collectionSettings.serverVersion == nil then
		collectionSettings.serverVersion = ''
	end

	if collectionSettings.coverImage == nil then
		collectionSettings.coverImage = ''
	end

	if collectionSettings.heroImage == nil then
		collectionSettings.heroImage = ''
	end

	if collectionSettings.pageCopyPlacement == nil then
		collectionSettings.pageCopyPlacement = 'above'
	end
	
	if collectionSettings.sidebarCopyPlacement == nil then
		collectionSettings.sidebarCopyPlacement = 'above'
	end

	if collectionSettings.tray2CopyPlacement == nil then
		collectionSettings.tray2CopyPlacement = 'above'
	end

	if collectionSettings.enableHeroImage == nil then
		collectionSettings.enableHeroImage = false
	end

	if collectionSettings.enablePassword == nil then
		collectionSettings.enablePassword = false
	end

	if collectionSettings.visibility == nil then
		collectionSettings.visibility = 'public'
	end

	if collectionSettings.username == nil or collectionSettings.username == '' then
		collectionSettings.username = 'guest'
	end

	if collectionSettings.password == nil or collectionSettings.password == '' then
		collectionSettings.password = 'guest'
	end

	if collectionSettings.allowEnablePassword == nil then
		collectionSettings.allowEnablePassword = true
	end
	
	if collectionSettings.hideFromAutoindex == nil then
		collectionSettings.hideFromAutoindex = false
	end
	
	if collectionSettings.hideFromSearch == nil then
		collectionSettings.hideFromSearch = false
	end
	
	if collectionSettings.showBreadcrumbs == nil then
		collectionSettings.showBreadcrumbs = true
	end
	
	if collectionSettings.breadcrumb == nil then
		collectionSettings.breadcrumb = ''
	end
end

function setAlbumSetDefaults(publishSettings, collectionSettings)
	setGenericDefaults(publishSettings, collectionSettings)

	if collectionSettings.ordering == nil then
		collectionSettings.ordering = 'title'
	end
end

function setAlbumDefaults(publishSettings, collectionSettings)
	setGenericDefaults(publishSettings, collectionSettings)

	if collectionSettings.itemsPerPage == nil then
		--collectionSettings.itemsPerPage = 0
		--collectionSettings.itemsPerPage=28
	end
	
	if collectionSettings.ordering == nil then
		collectionSettings.ordering = 'lightroom'
	end
	
	if collectionSettings.coverImage == nil then
		collectionSettings.coverImage = ''
	end

	if collectionSettings.enableMaps == nil then
		collectionSettings.enableMaps = true
	end

	if collectionSettings.enableCart == nil then
		collectionSettings.enableCart = true
	end
	
	if collectionSettings.enableCRG == nil then
		collectionSettings.enableCRG = false
	end

	if collectionSettings.crgClientMode == nil then
		collectionSettings.crgClientMode = 'managed'
	end

	--[[
	if collectionSettings.crgFeedbackProfile == nil then
		collectionSettings.crgFeedbackProfile = 'default'
	end
	]]--

	if collectionSettings.enableVideo == nil then
		logger:debug("enableVideo is true")
	end

	if collectionSettings.enableVideo == nil then
		--logger:debug("0enableVideo is nil")
		collectionSettings.enableVideo = true
	else
		--[[ if collectionSettings.enableVideo == true then
			logger:debug("0enableVideo is true")
		else
			logger:debug("0enableVideo is not true")
		end ]]--
	end

	if collectionSettings.pricing == nil or collectionSettings.pricing == '' then
		collectionSettings.pricing = 'default'
	end

	if collectionSettings.allowEnableCart == nil then
		collectionSettings.allowEnableCart = false --true
	end

	if collectionSettings.allowEnableCRG == nil then
		collectionSettings.allowEnableCRG = false --true
	end

	if collectionSettings.allowEnableVideo == nil then
		collectionSettings.allowEnableVideo = true
	end
	
	if collectionSettings.watermark == nil then
		collectionSettings.watermark = 'publisher'
	elseif collectionSettings.watermark ~= 'none' then
		local watermarks,indexed = TTGUtils.getWatermarks()
		if indexed[collectionSettings.watermark] == nil then
			collectionSettings.watermark = 'publisher'
		end
	end
	
	if collectionSettings.excludeWatermarksInThumbnails == nil then
		collectionSettings.excludeWatermarksInThumbnails = true
	end
	
	if collectionSettings.excludeWatermarksInPhotosForPurchase == nil then
		collectionSettings.excludeWatermarksInPhotosForPurchase = false
	end
end

function setSelectedTemplate(collectionSettings, templates)

	if #templates > 0 then

		local inspected = TTGUtils.inspect(templates);
		logger:debug('inspected in setSelectedTemplate: '..inspected)

		if collectionSettings.selectedTemplate ~= nil then
			logger:debug('selectedTemplate in setSelectedTemplate: '..collectionSettings.selectedTemplate)

			local foundTemplate = false
	
			for k,v in pairs(templates) do
				if collectionSettings.selectedTemplate ==  v.value then
					logger:debug('found a template');
					foundTemplate = true
					break
				end
			end

			logger:debug('found no template.');
			if foundTemplate == false then
				collectionSettings.selectedTemplate = ''
			end
		end

		if (collectionSettings.selectedTemplate == nil or collectionSettings.selectedTemplate == '') and #templates > 0 then
			collectionSettings.selectedTemplate = TTGUtils.firstValue(templates).value
		end
	else
		logger:debug('#templates == 0')
		collectionSettings.selectedTemplate = ''
	end
end

function getTemplateSettings(publishSettings, collectionSettings, remoteId, templateType)
	
	logger:debug('---------------------------- getTemplateSettings called ----------------------------')

	if #prefs.templates == 0 then
		logger:debug('no templates found')
		return
	end
		
	if (collectionSettings.selectedTemplate == nil or collectionSettings.selectedTemplate == '') then
		logger:debug('no template set')
		return
	end

	local settings, album, thumbnails, serverVersion = TTGApi.getSetupForTemplate(publishSettings, remoteId, templateType, collectionSettings.selectedTemplate)


	if serverVersion ~= nil then
		logger:debug('getTemplateSettings serverVersion: '..serverVersion)

		collectionSettings.serverVersion = serverVersion
	end

	if settings.heroImageEnabled ~= nil and settings.heroImageEnabled == 'yes'	then
		collectionSettings.enableHeroImage = true
	else
		collectionSettings.enableHeroImage = false
	end

	if settings.passwordEnabled ~= nil and settings.passwordEnabled == 'yes'	then
		collectionSettings.allowEnablePassword = true
	else
		collectionSettings.allowEnablePassword = false
		collectionSettings.enablePassword = false
	end
	
	if settings.mapsEnabled ~= nil and settings.mapsEnabled == 'yes'	then
		collectionSettings.allowEnableMaps = true
	else
		collectionSettings.allowEnableMaps = false
		collectionSettings.enableMaps = false
	end
	
	if settings.cartEnabled ~= nil and settings.cartEnabled == 'yes'	then
		collectionSettings.allowEnableCart = true
	else
		collectionSettings.allowEnableCart = false
		collectionSettings.enableCart = false
	end
	
	if settings.crgEnabled ~= nil and settings.crgEnabled == 'yes'	then
		-- We want to enable CRG for the case that the previous template was a non-CRG template, 
		-- but to maintain the enable-CRG state for changing from one CRG template to another
		if collectionSettings.allowEnableCRG == false then
			collectionSettings.enableCRG = true
		end
		
		collectionSettings.allowEnableCRG = true
	else
		collectionSettings.allowEnableCRG = false
		if collectionSettings.enableCRG == true then
			collectionSettings.enableCRG = false
		end
	end
	
	if collectionSettings.itemsPerPage == nil then
		if settings.itemsPerPage ~= nil then
			collectionSettings.itemsPerPage = math.floor(settings.itemsPerPage)
		else
			collectionSettings.itemsPerPage = 0
		end
	end
	
	if settings.videoEnabled ~= nil and settings.videoEnabled == 'yes'	then
		collectionSettings.enableVideo = true
	else
		collectionSettings.enableVideo = false
	end

	if album ~= nil then
		logger:debug('received album')

		-- Base Settings
		collectionSettings.title = album.title
		if album.type == 'album' then
			collectionSettings.description = album.description
		end
		collectionSettings.slug = album.slug
		collectionSettings.oldslug = album.slug
		collectionSettings.coverImage = album.cover_image

		if album.hero_image ~= nil then
			collectionSettings.heroImage = album.hero_image
		end

		collectionSettings.itemsPerPage = tonumber(album.items_per_page)
		collectionSettings.ordering = album.ordering

		-- Page Content
		collectionSettings.contentBlock = album.page_copy
		collectionSettings.pageCopyPlacement = album.page_copy_placement
		collectionSettings.sidebarCopy = album.sidebar_copy
		collectionSettings.sidebarCopyPlacement = album.sidebar_copy_placement
		collectionSettings.tray2Copy = album.tray2_copy
		collectionSettings.tray2CopyPlacement = album.tray2_copy_placement
		-- video id
		if album.type == 'album' then
			collectionSettings.videoID = album.video_id
		end

		-- Features
		if album.type == 'album' then
			collectionSettings.enableMaps = TTGUtils.yesNoToBoolean(album.enable_maps)
		end
		collectionSettings.visibility = album.visibility
		--we don't try to update this locally.  Saved access code is whatever is last saved, whether LR or BL
		--collectionSettings.password = album.password
		collectionSettings.showBreadcrumbs = TTGUtils.yesNoToBoolean(album.show_breadcrumbs)
		collectionSettings.breadcrumb = album.breadcrumb
		collectionSettings.hideFromAutoindex = TTGUtils.yesNoToBoolean(album.hide_from_autoindex)
		if album.type == 'album' then
			collectionSettings.hideFromSearch = TTGUtils.yesNoToBoolean(album.hide_from_search)
		end

		-- Integration
		if album.type == 'album' then
			collectionSettings.enableCart = TTGUtils.yesNoToBoolean(album.enable_cart)
			collectionSettings.pricing = album.pricing_scheme
			collectionSettings.enableCRG = TTGUtils.yesNoToBoolean(album.enable_crg);
			collectionSettings.crgFeedbackProfile = album.crg_feedback_profile_id
		end
	end

	if thumbnails ~= nil then
		collectionSettings.thumbnails = thumbnails
	end
end

function setGenericFeatureParameters(collectionSettings, parameters)
	parameters.enablePassword = TTGUtils.yesNo(collectionSettings.enablePassword)

	if collectionSettings.enablePassword == true then
		parameters.enablePassword = 'yes'
		if collectionSettings.username ~= nil and collectionSettings.password ~= nil then
			parameters.username = collectionSettings.username
			parameters.password = collectionSettings.password
		end
	else
		parameters.enablePassword = 'no'
	end

	parameters.visibility = collectionSettings.visibility

	if collectionSettings.visibility == 'protected' then
		if collectionSettings.password ~= nil then
			parameters.password = collectionSettings.password
		end
	end

	if collectionSettings.showBreadcrumbs == true then
		parameters.showBreadcrumbs = 'yes'
		if collectionSettings.breadcrumb ~= nil then
			parameters.breadcrumb = collectionSettings.breadcrumb
		end
	else
		parameters.showBreadcrumbs = 'no'
	end
	
	parameters.hideFromAutoindex = TTGUtils.yesNo(collectionSettings.hideFromAutoindex)
	parameters.hideFromSearch = TTGUtils.yesNo(collectionSettings.hideFromSearch)
end

function setAlbumSetFeatureParameters(collectionSettings, parameters)
	setGenericFeatureParameters(collectionSettings, parameters)
end

function setAlbumFeatureParameters(collectionSettings, parameters)
	setGenericFeatureParameters(collectionSettings, parameters)

	parameters.enableMaps = TTGUtils.yesNo(collectionSettings.enableMaps)

	if collectionSettings.enableCart == true then
		parameters.enableCart = 'yes'
		if collectionSettings.pricing ~= nil then
			parameters.pricing = collectionSettings.pricing
		end
	else
		parameters.enableCart = 'no'
	end

	if collectionSettings.allowEnableCRG == true then
		if collectionSettings.crgClientMode ~= nil then
			parameters.crgClientMode = collectionSettings.crgClientMode
		end
	end

	if collectionSettings.enableCRG == true then
		parameters.enableCRG = 'yes'
		if collectionSettings.crgFeedbackProfile ~= nil then
			parameters.crgFeedbackProfile = collectionSettings.crgFeedbackProfile
		end
	else
		parameters.enableCRG = 'no'
	end

	if collectionSettings.enableVideo == true then
		parameters.enableVideo = 'yes'
		if collectionSettings.videoID ~= nil then
			parameters.videoID = collectionSettings.videoID
		end
	else
		parameters.enableVideo = 'no'
	end
end

function updateCollectionURLs(propertyTable, children)
	for k,v in pairs(children) do
		if v:getRemoteId() == nil then
			logger:debug('updateCollectionURLs::remoteId: nil'	)
		else
			logger:debug('updateCollectionURLs::remoteId: '..v:getRemoteId())
		end
		
		if v.getChildren ~= nil then
			updateCollectionURLs(propertyTable, v:getChildren())
		end
		
		local result = TTGApi.getAlbum(propertyTable, v:getRemoteId())
		
		if result ~= nil and result.url ~= nil then
			logger:debug('remoteUrl: '..result.url)
			v:setRemoteUrl(result.url)
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------
--
--   Export service provider functions
--
-------------------------------------------------------------------------------------------------------------------------------------

function publishServiceProvider.sectionsForTopOfDialog(f, propertyTable)
  return TTGDialogs.topSections(f, propertyTable)
end

function publishServiceProvider.sectionsForBottomOfDialog(f, propertyTable)
  return TTGDialogs.bottomSections(f, propertyTable)
end

function publishServiceProvider.processRenderedPhotos( functionContext, exportContext )

	if exportContext.propertyTable.LR_isExportForPublish == nil or exportContext.propertyTable.LR_isExportForPublish == false then
		LrErrors.throwUserError('Photos can only be exported from the Publisher interface.')
	end

	--------------------------------------- Setup paramaters ---------------------------------------
	
	local exportSession = exportContext.exportSession
	local exportSettings = assert( exportContext.propertyTable )
	local nPhotos = exportSession:countRenditions()
	local publishedCollection = exportContext.publishedCollection
	local collectionInfo = publishedCollection:getCollectionInfoSummary()
	local collectionSettings = collectionInfo.collectionSettings
	local collectionId = publishedCollection:getRemoteId()
	local catalog = (import 'LrApplication').activeCatalog()
	local progressScope = exportContext:configureProgress { title = "Publishing "..nPhotos.." photos" }

	logger:debug('enableRetinaThumbnails: '..TTGUtils.inspect(collectionSettings))

	local parent = publishedCollection:getParent()
	
	if parent ~= nil then
		if parent:getRemoteId() == nil then
			LrErrors.throwUserError('The containing Album Set has not yet been created on the server.  Edit the parent Album Set to save it to the server before publishing to this Album.');
		end
	end

	if collectionId == nil then
		collectionId = createAlbum(publishedCollection, exportSettings, collectionSettings)
	end

	local count=0
	local templateSettings = TTGApi.getSetupForTemplate(exportSettings, collectionId, 'gallery', collectionSettings.selectedTemplate)

	local enableRetinaThumbnails = false

	if templateSettings.retinaThumbnailsEnabled ~= nil and templateSettings.retinaThumbnailsEnabled == 'yes' then
		enableRetinaThumbnails = true
	end

	--------------------------------------- Renditions ---------------------------------------

	local requiredRenditions = templateSettings.renditions.rendition
	local publishedRenditions = nil
	
	if exportSettings.pushMetadataOnly or exportSettings.publishThumbnailsOnly then
		publishedRenditions = TTGApi.getPublishedRenditions(exportSettings, collectionId)
	end
	
	local outlets = {}

	if templateSettings.outlets ~= nil then
		outlets = templateSettings.outlets.outlet
		table.insert(outlets, {name="ttg_capture_time", value="{YYYY}-{MM}-{DD} {HH}:{MIN}:{SS}"})
		table.insert(outlets, {name="ttg_latitude", value="{Latitude}"})
		table.insert(outlets, {name="ttg_longitude", value="{Longitude}"})
		table.insert(outlets, {name="ttg_keywords", value="{Keywords}"})
	end
	
	local numberOfRenditions = nPhotos*(#requiredRenditions)
	
	local photosToRender = {}
	local newPhotosCount = 0

	for i, rendition in exportContext.exportSession:renditions() do
		photoToRender = {photo=rendition.photo}
	
		if rendition.publishedPhotoId ~= nil then
			photoToRender.photoId = rendition.publishedPhotoId
		else
			photoToRender.photoId = ''
			newPhotosCount = newPhotosCount+1
		end
	
		photosToRender[i] = photoToRender
	end

	--------------------------------------- Main rendition loop ---------------------------------------

	local publisherWatermarkSetting = exportSettings.LR_useWatermark
	
	local watermarks = nil -- placeholder variable
	
	local embeddedMetadataOption = exportContext.propertyTable.LR_embeddedMetadataOption
	local minimizeEmbeddedMetadata = exportContext.propertyTable.LR_minimizeEmbeddedMetadata

	if embeddedMetadataOption ~= nil then
		logger:debug('embeddedMetadataOption: '..embeddedMetadataOption)
	end

	if minimizeEmbeddedMetadata then
		logger:debug('minimizeEmbeddedMetadata is true')
	end

	for photoCount,photoToRender in pairs(photosToRender) do
		local existingPhotoId = photoToRender.photoId

		for k,requiredRendition in pairs(requiredRenditions) do

			---------- Setup LRExportSession ----------
			
			-- Retain setting from Publisher dialogue for each photo
			exportContext.propertyTable.LR_embeddedMetadataOption = embeddedMetadataOption
			exportContext.propertyTable.LR_minimizeEmbeddedMetadata = minimizeEmbeddedMetadata

			local exportRenditionSettings = createBaseExportSettings(exportContext.propertyTable, requiredRendition)

			-- Handle watermarks
			
			logger:debug('requiredRendition.renditionPath: '..requiredRendition.renditionPath)

			local setupWatermarkId = nil
			
			if requiredRendition.watermark ~= nil and requiredRendition.watermark ~= '' then
				if watermarks == nil then
					watermarks,indexed = TTGUtils.getWatermarks()
				end

				setupWatermarkId = TTGUtils.getWatermarkIDForTitle(indexed, requiredRendition.watermark)
				
				if setupWatermarkId == nil then
					logger:debug('custom watermark of '..requiredRendition.watermark..' did not match and saved watermarks')
				end
			end
			
			if requiredRendition.watermark ~= nil and requiredRendition.watermark == '' then
				exportRenditionSettings.LR_useWatermark = false
			elseif setupWatermarkId ~= nil then
				logger:debug('using custom watermark: '..setupWatermarkId)
				exportRenditionSettings.LR_useWatermark = true
				exportRenditionSettings.LR_watermarking_id = setupWatermarkId
			elseif collectionSettings.watermark == 'publisher' then
				exportRenditionSettings.LR_useWatermark = publisherWatermarkSetting
			elseif collectionSettings.watermark == 'none' then
				exportRenditionSettings.LR_useWatermark = false
			else
				local watermarks,indexed = TTGUtils.getWatermarks()

				if indexed[collectionSettings.watermark] == nil then
					logger:debug('watermark is nil')
					exportRenditionSettings.LR_useWatermark = publisherWatermarkSetting
				else
					logger:debug('watermark is NOT nil')
					exportRenditionSettings.LR_useWatermark = true
					exportRenditionSettings.LR_watermarking_id = collectionSettings.watermark
				end
			end
			
			if requiredRendition.renditionPath == 'thumbnails' or requiredRendition.renditionPath == 'thumbnailsMobile' or requiredRendition.renditionPath == 'thumbnails-for-mobile' then
				if collectionSettings.excludeWatermarksInThumbnails == nil or collectionSettings.excludeWatermarksInThumbnails then
					exportRenditionSettings.LR_useWatermark = false
				end
				
				if enableRetinaThumbnails then
					if exportRenditionSettings.LR_size_maxWidth ~= nil then
						exportRenditionSettings.LR_size_maxWidth = exportRenditionSettings.LR_size_maxWidth * 2
					end
					
					if exportRenditionSettings.LR_size_maxHeight ~= nil then
						exportRenditionSettings.LR_size_maxHeight = exportRenditionSettings.LR_size_maxHeight * 2
					end
				end
				
			elseif requiredRendition.renditionPath == 'photos-for-purchase' then 
				if collectionSettings.excludeWatermarksInPhotosForPurchase ~= nil and collectionSettings.excludeWatermarksInPhotosForPurchase then
					exportRenditionSettings.LR_useWatermark = false
				end
			end

			local exportParams = {
				photosToExport = {photoToRender.photo},
				exportSettings = exportRenditionSettings
			}

			local exportRenditionSession = LrExportSession( exportParams )	

			for i, rendition in exportRenditionSession:renditions() do

				--local needsRendition = publishedRenditions == nil or not hasPublishedRendition( photoToRender.photoId, requiredRendition.renditionPath, publishedRenditions )
				--if photoToRender.photoId == '' or exportSettings.pushMetadataOnly == false or needsRendition then

				-- We need a rendition if it has never been published, or if it has been published and pushMetadataOnly is off
				-- and photo is a thumbnail if publishThumbnailsOnly
				local needsRendition = photoToRender.photoId == '' or publishedRenditions == nil or not hasPublishedRendition( photoToRender.photoId, requiredRendition.renditionPath, publishedRenditions )

				if not needsRendition and not exportSettings.pushMetadataOnly then
					local isThumbnail = TTGUtils.stringStartsWithString(requiredRendition.renditionPath, 'thumbnail')
					needsRendition = isThumbnail or not exportSettings.publishThumbnailsOnly
				end

				if needsRendition then

					local filePath = rendition.destinationPath

					---------- Render and check file ----------
					
					local success, pathOrMessage = rendition:waitForRender()
					local fileName = LrPathUtils.leafName( filePath )
					fileName = TTGUtils.sanitizeFileName(fileName)

					if LrFileUtils.fileAttributes(filePath) == nil then
						LrErrors.throwUserError('Lightroom can not find the original photo.  If the photo resides on external storage, please ensure that the drive is connected.')
						logger:debug('unable to create rendition for '..filePath..' - fileAttributes are nil')
					end	

					local fileAttributes = LrFileUtils.fileAttributes(filePath);
					
					if fileAttributes.fileSize == nil or fileAttributes.fileSize == 0 then
						if fileAttributes.fileSize == nil then
							logger:debug('unable to create rendition for '..filePath..' - fileSize is nil')
						else
							logger:debug('unable to create rendition for '..filePath..' - fileSize is 0')
						end

						LrErrors.throwUserError('The rendition could not be created.  Please ensure that the original file name is accessible, especially when stored on external or network storage.')
					end

					---------- Prepare parameters ---------- 

					local parameters = {
						renditionPath=requiredRendition.renditionPath, 
						fileSize=LrFileUtils.fileAttributes(filePath).fileSize, 
						isVirtualCopy=rendition.photo:getRawMetadata('isVirtualCopy'),
						existingPhotoId=existingPhotoId
					}
				
					parameters.width,parameters.height = calculateDimensions(rendition.photo, requiredRendition)

					photoId, webUrl = TTGApi.uploadRendition(collectionId, photoToRender.photoId, filePath, fileName, parameters, exportRenditionSettings)
					photoToRender.photoId = photoId

					LrFileUtils.delete(rendition.destinationPath)
		
				end -- end if exportSettings.pushMetadataOnly

				count = count+1
				progressScope:setPortionComplete( count / numberOfRenditions )

			end
		end
		
		--------------------------------------- Handle TTG Metadata and save to catalog ---------------------------------------
		
		for i, rendition in exportContext.exportSession:renditions() do
		
			if i == photoCount and photoToRender.photoId ~= nil then
				
				--local iOutlets = TTGUtils.inspect(outlets)
				local metadata,count = MetadataHandler.createMetadataForPhoto(photoToRender.photo, outlets, catalog:getLabelMapToColorName())

				if count > 0 then
					TTGApi.saveMetadata(collectionId, photoToRender.photoId, metadata, exportSettings)
				end
				
				rendition:recordPublishedPhotoId(photoToRender.photoId)
				
				if webUrl then 
					rendition:recordPublishedPhotoUrl(webUrl)
				end
			end
		end

	end -- end of photoToRender loop

	if exportSettings.openAlbumAfterExport then
		if webUrl then 
			LrHttp.openUrlInBrowser( webUrl ) 
		end
	end

	--------------------------------------- Cleanup orphaned renditions ---------------------------------------
	
	if exportSettings.pushMetadataOnly ~= true then
		local renditionPaths = ''
	
		for k,v in pairs(requiredRenditions) do
			if renditionPaths ~= '' then
				renditionPaths = renditionPaths..'|'
			end
		
			renditionPaths = renditionPaths..v.renditionPath
		end
		
		TTGApi.cleanupPartialUploads(collectionId, renditionPaths, exportSettings)
	end

	TTGApi.resetHasUserOrder(collectionId, exportSettings)
	
end

-------------------------------------------------------------------------------------------------------------------------------------
--
--   Publisher service provider functions
--
-------------------------------------------------------------------------------------------------------------------------------------

function publishServiceProvider.getCollectionBehaviorInfo( publishSettings )
	return {
		defaultCollectionName = "Default Album",
		defaultCollectionCanBeDeleted = true,
		canAddCollection = true,
		maxCollectionSetDepth = 20,	}
end

--[[
function publishServiceProvider.metadataThatTriggersRepublish( publishSettings )
	return {
		'default'
	}
end
]]--

function publishServiceProvider.viewForCollectionSetSettings( f, publishSettings, info )
	local collectionSettings = assert( info.collectionSettings )
	
	logger:debug("collectionSettings: "..TTGUtils.inspect(collectionSettings))

	LrTasks.startAsyncTask( function()
		local selectedTemplate = ''

		if info.publishedCollection ~= nil then
			remoteId = info.publishedCollection:getRemoteId()
		else
			remoteId = nil
		end

		prefs.autoindexTemplates, selectedTemplate = TTGApi.fetchAutoindexTemplates(publishSettings, remoteId)

		if selectedTemplate ~= '' then
			collectionSettings.selectedTemplate = selectedTemplate
		end

		setSelectedTemplate(collectionSettings, prefs.autoindexTemplates)
		getTemplateSettings(publishSettings, collectionSettings, remoteId, 'autoindex')
	end)

	collectionSettings:addObserver( 'selectedTemplate', function( properties, key, newValue )
		-- this gets called immediately, so no need to getTemplateSettings elsewhere
		collectionSettings.selectedTemplate = newValue
		LrTasks.startAsyncTask( function()
			getTemplateSettings(publishSettings, collectionSettings, remoteId, 'autoindex')
		end )
	end )

	setAlbumSetDefaults(publishSettings, collectionSettings)

	return TTGDialogs.albumSetSettingsDialog(f, publishSettings, info)
end

function publishServiceProvider.viewForCollectionSettings( f, publishSettings, info )
	local collectionSettings = assert( info.collectionSettings )
	
	LrTasks.startAsyncTask( function()
		local selectedTemplate = ''

		if info.publishedCollection ~= nil then
			remoteId = info.publishedCollection:getRemoteId()
		else
			remoteId = nil
		end

		prefs.templates, selectedTemplate, prefs.cartPricingSchemes, prefs.crgFeedbackProfiles = TTGApi.fetchTemplates(publishSettings, remoteId)

		if selectedTemplate ~= '' then
			collectionSettings.selectedTemplate = selectedTemplate
		end

		local inspected = TTGUtils.inspect(prefs.templates)
		logger:debug('inspected templates: '..inspected)

		if collectionSettings.crgFeedbackProfile == nil and #prefs.crgFeedbackProfiles > 0 then
			local firstProfile = prefs.crgFeedbackProfiles[1]
			logger:debug('firstProfile value: '..firstProfile.value)
			collectionSettings.crgFeedbackProfile = firstProfile.value
		end
		
		setSelectedTemplate(collectionSettings, prefs.templates)
		getTemplateSettings(publishSettings, collectionSettings, remoteId, 'gallery')
	end)

	collectionSettings:addObserver( 'selectedTemplate', function( properties, key, newValue )
		-- this gets called immediately, so no need to getTemplateSettings elsewhere

		logger:debug('setting selected Template1')
		logger:debug('setting selected Template1: '..newValue)
		collectionSettings.selectedTemplate = newValue

		LrTasks.startAsyncTask( function()
			getTemplateSettings(publishSettings, collectionSettings, remoteId, 'gallery')
		end )
	end )

	setAlbumDefaults(publishSettings, collectionSettings)

	return TTGDialogs.albumSettingsDialog(f, publishSettings, info )
end

function publishServiceProvider.endDialogForCollectionSetSettings( publishSettings, info )
	local collectionSettings = info.collectionSettings

	logger:debug("endDialogForCollectionSetSettings.name: "..info.name)

	if collectionSettings.title == nil or collectionSettings.title == '' then
		collectionSettings.title = info.name
	end

	logger:debug("endDialogForCollectionSetSettings collectionSettings.title: "..collectionSettings.title)

	collectionSettings.slug = TTGUtils.createSlugFromString(collectionSettings.slug, {preserve = publishSettings.preserveCapitalizationInAlbumSlugs})

	if collectionSettings.slug == nil or collectionSettings.slug == '' then
		collectionSettings.slug = TTGUtils.createSlugFromString(info.name, {preserve = publishSettings.preserveCapitalizationInAlbumSlugs})
	end
end

function publishServiceProvider.endDialogForCollectionSettings( publishSettings, info )
	local collectionSettings = info.collectionSettings

	--[[
	local catalog = (import 'LrApplication').activeCatalog()

	catalog:withWriteAccessDo( "albumDefaults", function( context ) 
		local service = info.publishService
		local albumDefaults = catalog:getPropertyForPlugin(_PLUGIN, service:getName()..'-albumDefaults')
		
		logger:debug('fieldName: '..service:getName()..'-albumDefaults')
		
		if albumDefaults ~= nil then
			logger:debug('==================== endDialogForCollectionSettings, albumDefaults: '..TTGUtils.inspect(albumDefaults))
		end
	end, writeTimeout ) 
	]]--
	
	logger:debug('endDialogForCollectionSettings, info.name: '..info.name)

	if collectionSettings.title == nil or collectionSettings.title == '' then
		collectionSettings.title = info.name
	end

	collectionSettings.slug = TTGUtils.createSlugFromString(collectionSettings.slug,  {preserve = publishSettings.preserveCapitalizationInAlbumSlugs})

	if collectionSettings.slug == nil or collectionSettings.slug == '' then
		collectionSettings.slug = TTGUtils.createSlugFromString(info.name,  {preserve = publishSettings.preserveCapitalizationInAlbumSlugs})
	end
	
	if collectionSettings.selectedTemplate == nil or collectionSettings.selectedTemplate == '' then
		LrErrors.throwUserError('The template has not been selected.')
	end
	
	--[[
	catalog:withWriteAccessDo( "albumDefaults", function( context ) 
		local service = info.publishService
		catalog:setPropertyForPlugin(_PLUGIN, service:getName()..'-albumDefaults', 'hello')
	end, writeTimeout ) 
	]]--
	
end

function publishServiceProvider.updateCollectionSetSettings( publishSettings, info )
	logger:debug("called updateCollectionSetSettings")
	local collectionSettings = info.collectionSettings

	-- Handle slug
	if (collectionSettings.oldslug == nil) then
		if TTGApi.albumExists(publishSettings, getParentId(info), collectionSettings.slug) == "exists" then
			LrDialogs.message( 'Duplicate album set found', 'This album set will have a number automatically appended to the slug.', 'info' )
		end
	elseif (collectionSettings.slug ~= collectionSettings.oldslug)
			and TTGApi.albumExists(publishSettings, getParentId(info), collectionSettings.slug) == "exists" then

		what = LrDialogs.confirm( 'Duplicate album set found', 'If you proceed, this album set will have a number automatically appended to the slug.', 'Continue', 'Cancel' )
		logger:debug("what: "..what)
		if what == 'ok' then
			-- auto rename
			collectionSettings.slug = TTGApi.createAutoSlug(publishSettings, collectionSettings.slug)
		else
			LrErrors.throwUserError('The album set was not updated');
		end
	end

	local publishedCollection = info.publishedCollection
	local catalog = (import 'LrApplication').activeCatalog()

	if publishedCollection:getRemoteId() == nil then  -- CREATE ALBUM SET
		createAlbumSet(publishedCollection, publishSettings, collectionSettings)		
	else  -- UPDATE ALBUM SET
		local parameters = {slug=collectionSettings.slug, title=collectionSettings.title, description=collectionSettings.description, template=collectionSettings.selectedTemplate, blockContent=collectionSettings.contentBlock, pageCopyPlacement=collectionSettings.pageCopyPlacement, sidebarCopy=collectionSettings.sidebarCopy, sidebarCopyPlacement=collectionSettings.sidebarCopyPlacement, tray2Copy=collectionSettings.tray2Copy, tray2CopyPlacement=collectionSettings.tray2CopyPlacement, coverImage=collectionSettings.coverImage, heroImage=collectionSettings.heroImage, ordering=collectionSettings.ordering}
		setAlbumSetFeatureParameters(collectionSettings, parameters)
		parameters.id = publishedCollection:getRemoteId();
		local parent = publishedCollection:getParent()
		if (parent ~= nil) then
			parameters.parent = parent:getRemoteId()
		end

		id,url,slug = TTGApi.updateAlbumSet(publishSettings, parameters)

		catalog:withWriteAccessDo( "updateUrl", function( context ) 
			publishedCollection:setRemoteId(id) -- in case id has changed
			publishedCollection:setRemoteUrl(url)

			if collectionSettings.slug ~= collectionSettings.oldslug then
				if info.publishedCollection.getChildren ~= nil then
					updateCollectionURLs(publishSettings, publishedCollection:getChildren())
				end
			end

		end, writeTimeout ) 
	end

	return true;
end

function createAlbum(publishedCollection, publishSettings, collectionSettings)
	local parameters = 	{slug=collectionSettings.slug, title=collectionSettings.title, description=collectionSettings.description, template=collectionSettings.selectedTemplate, blockContent=collectionSettings.contentBlock, pageCopyPlacement=collectionSettings.pageCopyPlacement, sidebarCopy=collectionSettings.sidebarCopy, sidebarCopyPlacement=collectionSettings.sidebarCopyPlacement, tray2Copy=collectionSettings.tray2Copy, tray2CopyPlacement=collectionSettings.tray2CopyPlacement, itemsPerPage=collectionSettings.itemsPerPage, ordering=collectionSettings.ordering}

	setAlbumFeatureParameters(collectionSettings, parameters)
	local parent = publishedCollection:getParent()
	
	if (parent ~= nil) then
		parameters.parent = parent:getRemoteId()
		--if (parent:getRemoteId() == nil) then
		--	publishedCollection:setParent(nil)
		--end
	end

	local id,url,slug = TTGApi.createAlbum(publishSettings, parameters)
	local catalog = (import 'LrApplication').activeCatalog()
	
	catalog:withWriteAccessDo( "updateUrl", function( context ) 
		publishedCollection:setRemoteId(id)
		publishedCollection:setRemoteUrl(url)
		collectionSettings.slug = slug
		publishedCollection:setCollectionSettings(collectionSettings)
	end, writeTimeout )
	
	return id
end

function createAlbumSet(publishedCollection, publishSettings, collectionSettings)
	local parameters = {slug=collectionSettings.slug, title=collectionSettings.title, description=collectionSettings.description, template=collectionSettings.selectedTemplate, blockContent=collectionSettings.contentBlock, pageCopyPlacement=collectionSettings.pageCopyPlacement, sidebarCopy=collectionSettings.sidebarCopy, sidebarCopyPlacement=collectionSettings.sidebarCopyPlacement, tray2Copy=collectionSettings.tray2Copy, tray2CopyPlacement=collectionSettings.tray2CopyPlacement, ordering=collectionSettings.ordering}
	
	setAlbumSetFeatureParameters(collectionSettings, parameters)
	local parent = publishedCollection:getParent()
	if (parent ~= nil) then
		parameters.parent = parent:getRemoteId()
	end

	local id,url,slug = TTGApi.createAlbumSet(publishSettings, parameters)
	local catalog = (import 'LrApplication').activeCatalog()

	logger:debug('created album set with id: '..id)
	catalog:withWriteAccessDo( "updateUrl", function( context ) 
		publishedCollection:setRemoteId(id)
		publishedCollection:setRemoteUrl(url)
		collectionSettings.slug = slug
		publishedCollection:setCollectionSetSettings(collectionSettings)
	end, writeTimeout )

	return id

end

function publishServiceProvider.updateCollectionSettings( publishSettings, info )
	logger:debug("called updateCollectionSettings")
	local collectionSettings = info.collectionSettings

	-- Handle slug
	if (collectionSettings.oldslug == nil) then
		if TTGApi.albumExists(publishSettings, getParentId(info), collectionSettings.slug) == "exists" then
			LrDialogs.message( 'Duplicate gallery found', 'This gallery will have a number automatically appended to the slug.', 'info' )
		end
	elseif (collectionSettings.slug ~= collectionSettings.oldslug)
			and TTGApi.albumExists(publishSettings, getParentId(info), collectionSettings.slug) == "exists" then

		what = LrDialogs.confirm( 'Duplicate gallery found', 'If you proceed, this gallery will have a number automatically appended to the slug.', 'Continue', 'Cancel' )
		logger:debug("what: "..what)
		if what == 'ok' then
			-- auto rename
			collectionSettings.slug = TTGApi.createAutoSlug(publishSettings, collectionSettings.slug)
		else
			LrErrors.throwUserError('The album was not updated');
		end
	end

	local publishedCollection = info.publishedCollection
	local catalog = (import 'LrApplication').activeCatalog()

	if publishedCollection:getRemoteId() == nil then  -- CREATE ALBUM
		createAlbum(publishedCollection, publishSettings, collectionSettings)
	else  -- UPDATE ALBUM
		local parameters = {slug=collectionSettings.slug, title=collectionSettings.title, description=collectionSettings.description, template=collectionSettings.selectedTemplate, blockContent=collectionSettings.contentBlock, pageCopyPlacement=collectionSettings.pageCopyPlacement, sidebarCopy=collectionSettings.sidebarCopy, sidebarCopyPlacement=collectionSettings.sidebarCopyPlacement, tray2Copy=collectionSettings.tray2Copy, tray2CopyPlacement=collectionSettings.tray2CopyPlacement, coverImage=collectionSettings.coverImage, heroImage=collectionSettings.heroImage, itemsPerPage=collectionSettings.itemsPerPage, ordering=collectionSettings.ordering}
		setAlbumFeatureParameters(collectionSettings, parameters)
		parameters.id = publishedCollection:getRemoteId();

		local parent = publishedCollection:getParent()
		if (parent ~= nil and parent:getRemoteId() ~= nil) then
			parameters.parent = parent:getRemoteId()
		end

		id,url,slug = TTGApi.updateAlbum(publishSettings, parameters)
		-- info.collectionSettings.slug = slug .. 'k'

		catalog:withWriteAccessDo( "updateUrl", function( context ) 
			publishedCollection:setRemoteId(id) -- in case id has changed
			publishedCollection:setRemoteUrl(url)

			local publishedPhotos = publishedCollection:getPublishedPhotos()

			for i=1,#publishedPhotos,1 do
				publishedPhotos[i]:setRemoteUrl(url)
			end

		end, writeTimeout ) 
	end

	return true;
end

function publishServiceProvider.deletePhotosFromPublishedCollection( publishSettings, arrayOfPhotoIds, deletedCallback, localCollectionId )
	
	logger:debug("deletePhotosFromPublishedCollection called for collection: "..localCollectionId);
		
	for i=1,#arrayOfPhotoIds,1 do
		local id = arrayOfPhotoIds[i]
		
		logger:debug("deleting: "..id)
		
		if TTGApi.deletePhoto(publishSettings, id) then
			deletedCallback(id)
		end
	end
end

function publishServiceProvider.renamePublishedCollection( publishSettings, info )
end

function publishServiceProvider.deletePublishedCollection( publishSettings, info )

	logger:debug("publishServiceProvider.deletePublishedCollection called")

	local collectionSettings = info.collectionSettings
	local publishedCollection = info.publishedCollection

	TTGApi.deleteAlbum(publishSettings, info.remoteId)
end

function publishServiceProvider.imposeSortOrderOnPublishedCollection( publishSettings, info, remoteIdSequence )
	logger:debug('imposeSortOrderOnPublishedCollection')

	if info.remoteCollectionId ~= nil then
		TTGApi.setPhotoOrder(info.remoteCollectionId, remoteIdSequence, publishSettings)
	else
		logger:debug('imposeSortOrderOnPublishedCollection: remoteCollectionId is nil')
	end

end

function publishServiceProvider.reparentPublishedCollection( publishSettings, info )
	logger:debug('reparenting')
	
	local collectionId = info.remoteId
	
	if collectionId == nil then
		local publishedCollection = info.publishedCollection
		
		if publishedCollection.getCollectionInfoSummary ~= nil then
			logger:debug('This is an Album that does not yet exist on the server')
			local collectionInfoSummary = info.publishedCollection:getCollectionInfoSummary()
			collectionId = createAlbum(info.publishedCollection, publishSettings, collectionInfoSummary.collectionSettings)
		elseif publishedCollection.getCollectionSetInfoSummary ~= nil then
			logger:debug('This is an Album Set that does not yet exist on the server')
			local collectionInfoSummary = info.publishedCollection:getCollectionSetInfoSummary()
			collectionId = createAlbumSet(info.publishedCollection, publishSettings, collectionInfoSummary.collectionSettings)
		end
	end

	local parents = info.parents

	for k,v in pairs(parents) do
		if collectionId == v.remoteCollectionId then
			LrErrors.throwUserError('An album set can not be moved to one of child album sets');
		end
	end
	
	local parentId = ''
	
	if #parents == 0 then
		logger:debug('moving '..collectionId..' to ROOT')
	else
		local parent = parents[#parents]
		parentId = parent.remoteCollectionId
		
		if parentId == nil then
			LrErrors.throwUserError('The Album Set you are trying to move to has not yet been created on the server.  Edit the destination Album Set to save it to the server.');
			-- We don't have an LrPublishedCollectionSet to work with here
			--local collectionInfoSummary = info.publishedCollection:getCollectionInfoSummary()
			--local collectionSettings = collectionInfoSummary.collectionSettings
			--parentId = createAlbumSet(info.publishedCollection, publishSettings, collectionSettings)
		else
			logger:debug('moving '..collectionId..' to '..parents[#parents].remoteCollectionId)
		end
	end
	
	id,url = TTGApi.reparent(publishSettings, collectionId, parentId)
	logger:debug('new url: '..url)

	local publishedCollection = info.publishedCollection

	local catalog = (import 'LrApplication').activeCatalog()
	catalog:withWriteAccessDo( "updateUrl", function( context ) 
		logger:debug('new url: '..url)
		--publishedCollection:setRemoteId(id)
		publishedCollection:setRemoteUrl(url)
		
		if info.publishedCollection.getChildren ~= nil then
			updateCollectionURLs(publishSettings, publishedCollection:getChildren())
		end
		
	end, writeTimeout ) 
end

function publishServiceProvider.validatePublishedCollectionName( proposedName )
	if string.match(proposedName, "^[%w%_%-% ]+$") then
		return true
	else
		return false, "Please use only letters, number, spaces or '-_"
	end
end

-- NOT YET IMPLEMENTED
function publishServiceProvider.getCommentsFromPublishedCollection( publishSettings, arrayOfPhotoInfo, commentCallback )
end

-- NOT YET IMPLEMENTED
function publishServiceProvider.getRatingsFromPublishedCollection( publishSettings, arrayOfPhotoInfo, ratingCallback )
end

return publishServiceProvider