local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrStringUtils = import 'LrStringUtils'

local LrLogger = import 'LrLogger'
local logger = LrLogger("ttg")
logger:enable( "logfile" )

TTGUtils = {}

function TTGUtils.yesNo(val)
	if val == nil then
		return 'no'
	elseif val == true then
		return 'yes'
	else
		return 'no'
	end
end

function TTGUtils.yesNoToBoolean(val)
	if val == nil then
		return false
	elseif val == 'yes' then
		return true
	else
		return false
	end
end

function TTGUtils.round(x)
	if x%2 ~= 0.5 then
		return math.floor(x+0.5)
	else
		return x-0.5
	end
end

function TTGUtils.firstValue(collection)
	for k,v in pairs(collection) do
		return v
	end

	return nil
end

function TTGUtils.set (list)
  local set = {}
  for _, l in ipairs(list) do set[""..l] = true end
  return set
end

-- Find position of last occurrence of a char in a string
-- by Fernando P. Garcia
function strrpos(haystack, needle, offset)
  offset = offset or 0
  local last
  local pos = 1
  repeat
    last = pos
    offset = last
    pos = string.find(haystack, needle, offset + 1)
  until pos == nil
  return last
end

function TTGUtils.split(str, sep)
	return {str:match((str:gsub("[^"..sep.."]*"..sep, "([^"..sep.."]*)"..sep)))}
end

function TTGUtils.trim(s)
	s = string.gsub(s, "^%s+", "")
	s = string.gsub(s, "%s+$", "")
	return s
end

function TTGUtils.stringStartsWithString(string, start)
	return string.sub(string, 1, string.len(start)) == start
end

function TTGUtils.addTrailingSlashIfNeeded(s)
	if s == nil then
		return ""
	end

	local delim_to = strrpos(s, 'm=publisher', 0)

	if delim_to > 1 then
		return s
	end

	local delim_to = strrpos(s, 'publisher', 0)
	
	logger:debug('length: '..#s)
	logger:debug('delim_to: '..delim_to)
	
	if #s - delim_to == 8 then
		return s..'/'
	else
		return s
	end
	
end

--function TTGUtils.createSlugFromString(s, preserve, d)
function TTGUtils.createSlugFromString(s, options)
	if s == nil or s == '' then
		if options ~= nil and options['default'] ~= nil then
			return options['default']
		else
			return ""
		end
	end

	-- Attempt to prevent something like "1st gallery" from being converted to "true"
	s = tostring(s)

	s = string.gsub(s, "%s+", "-")
	s = string.gsub(s, "_", "UNDERSCORE")
	if options ~= nil and options['allowSlash'] ~= nil and options['allowSlash'] == true then
		s = string.gsub(s, "/", "FORWARDSLASH")
	end
	s = string.gsub(s, "%p+", "-")
	s = string.gsub(s, "[^%w%s\-]", "")
	s = string.gsub(s, "UNDERSCORE", "_")
	if options ~= nil and options['allowSlash'] ~= nil and options['allowSlash'] == true then
		s = string.gsub(s, "FORWARDSLASH", "/")
	end
	s = string.gsub(s, "^\-", "")
	s = string.gsub(s, "\-$", "")
	
	if options == nil or options['preserve'] == nil or options['preserve'] == false then
		s = string.lower(s)
	end
	
	return s
end

function TTGUtils.sanitizeFileName(fileName)
	if fileName == nil or fileName == '' then
		return ""
	end
	
	fileName = string.gsub(fileName, "%s+", "-")
	fileName = string.gsub(fileName, "&+", "-")
	fileName = string.gsub(fileName, "?+", "-")
	fileName = string.gsub(fileName, "#+", "-")
	fileName = string.gsub(fileName, "\++", "-")
	fileName = string.gsub(fileName, "%(", "")
	fileName = string.gsub(fileName, "%)", "")

	return fileName
end

function TTGUtils.inspect(obj)
	if obj==nil then return "nil" end
	if obj==true then return "true" end
	if obj==false then return "false" end
	if type(obj)=="string" then return obj end
	--local json = require "json"
	return JSON:encode({obj})
end
 
function TTGUtils.dumpvar(data)
    -- cache of tables already printed, to avoid infinite recursive loops
    local tablecache = {}
    local buffer = ""
    local padder = "    "

    local function _dumpvar(d, depth)
        local t = type(d)
        local str = tostring(d)
        if (t == "table") then
				if (tablecache[str]) then
					-- table already dumped before, so we dont
					-- dump it again, just mention it
					buffer = buffer.."<"..str..">\n"
				else
					tablecache[str] = (tablecache[str] or 0) + 1
					buffer = buffer.." { "
					for k, v in pairs(d) do
					    buffer = buffer..string.rep(padder, depth+1).."["..k.."] => "
					    _dumpvar(v, depth+1)
					end
					buffer = buffer..string.rep(padder, depth).."}\n"
				end
        elseif (t == "number") then
            buffer = buffer..str.."\n"
        else
            --buffer = buffer.."\""..str.."\"\n"
            buffer = buffer.."\""..str.."\""
        end
    end
    _dumpvar(data, 0)
    return buffer
end

function TTGUtils.encodeTable(table, htmlAllowedFields)
	local encoded = {}
	
	for k,v in pairs(table) do
		if v == nil or type(v) ~= "string" then
			--logger:debug('not encoded')
			encoded[k] = v
		else
			encoded[k] = TTGUtils.encodeEntities(v, htmlAllowedFields[k])
			--logger:debug('encoded: '..encoded[k])
		end
	end
	
	-- temp = temp.."propT:"..k.."=>"..tostring(v).."\n" 
	
	return encoded
end

--https://stackoverflow.com/questions/2705793/how-to-get-number-of-entries-in-a-lua-table
function TTGUtils.tablelength(T)
	local count = 0
	for _ in pairs(T) do count = count + 1 end
	return count
end


-- taken and adapted from http://lua-users.org/files/wiki_insecure/users/PhiLho/EncodeEntities.lua

TTGUtils.urlEntities =
{
	['{'] = "%%7B",
	['}'] = "%%7D",
	['&'] = "%%26",
	['+'] = "%%2B",
	['%%'] = "%%25",
	-- ["\n"] = "X",
}


TTGUtils.entitiesAllowHTML =
{
	-- ['"'] = "&quot;";
}

TTGUtils.entitiesRemoveHTML =
{
	['<'] = "&lt;",
	['>'] = "&gt;",
	['"'] = "&quot;";
}

function TTGUtils.encodeEntities(string, allowHTML)
	if string == nil or type(string) ~= "string" then
		return ''
	end

	local encodedString = string

	encodedString = string.gsub(encodedString, '&', '&amp;')

	local entities = nil
	
	if allowHTML then 
		entities = TTGUtils.entitiesAllowHTML
	else
		entities = TTGUtils.entitiesRemoveHTML
	end

	for char,entity in pairs(entities) do
		encodedString = string.gsub(encodedString, char, entity)
	end

	return encodedString
end

function TTGUtils.urlEncode(string)
	if string == nil or type(string) ~= "string" then
		return ''
	end

	local encodedString = string

	for char,entity in pairs(TTGUtils.urlEntities) do
		encodedString = string.gsub(encodedString, char, entity)
	end

	return encodedString
end

function TTGUtils.getFormattedCaptureTime(photo)
	local time = photo:getRawMetadata('dateTimeOriginal')
       or photo:getRawMetadata('dateTimeDigitized')
       or photo:getRawMetadata('dateTime')
	
	local val = LrDate.timeToUserFormat(time, '%Y-%m-%d %H:%M:%S')
	
	return val
end


function TTGUtils.getWatermarkIDForTitle(watermarks, lookup)
	for id,title in pairs(watermarks) do
		if lookup == title then
			return id
		end
	end
	
	return nil
end

function TTGUtils.getWatermarks()
	local watermarks = {}
	local count = 1

	--logger:debug('appData: '..LrPathUtils.getStandardFilePath('appData'))
	local watermarksPrefsPath = LrPathUtils.getStandardFilePath('appData')..'/Watermarks'

	-- logger:debug('Reading watermarks from: '..watermarksPrefsPath)

	for filePath in LrFileUtils.files( watermarksPrefsPath ) do 
		local pos = string.find(filePath, 'lrtemplate')

		if pos then
			--logger:debug('Reading watermark at: '..filePath)
	
			local wContents = LrFileUtils.readFile(filePath)
		
			if wContents then
				local f = loadstring(wContents)
			
				if f then
					s = nil
					f()
	
					if s then
						--logger:debug('Watermark added: '..s.id..' => '..s.title)
						table.insert(watermarks, {s.id, 'Global: '..s.title })
						count = count + 1
					end
				end
			end
		end
	end

	local catalog = (import 'LrApplication').activeCatalog()

	local catalogPath = catalog:getPath()
	-- logger:debug('Reading watermarks from 1: '..catalogPath)
	catalogWatermarksPath = TTGUtils.dirname(catalogPath)..'/Lightroom Settings/Watermarks'

	-- logger:debug('Reading watermarks from 2: '..catalogWatermarksPath)

	for filePath in LrFileUtils.files( catalogWatermarksPath ) do
		local pos = string.find(filePath, 'lrtemplate')

		if pos then
			-- logger:debug('Reading watermark at: '..filePath)

			local wContents = LrFileUtils.readFile(filePath)

			if wContents then
				local f = loadstring(wContents)

				if f then
					s = nil
					f()

					if s then
						--logger:debug('Watermark added: '..s.id..' => '..s.title)
						table.insert(watermarks, {s.id, 'Catalog: '..s.title })
						count = count + 1
					end
				end
			end
		end
	end

	-- local inspected = TTGUtils.inspect(watermarks)
	-- logger:debug('watermarks: '..inspected)

	function compare(a,b)
		return LrStringUtils.compareStrings(a[2], b[2])
	end

	table.sort(watermarks, compare)

	local indexed = {}

	for i,watermark in pairs(watermarks) do
		indexed[watermark[1]] = watermark[2]
	end

	return watermarks,indexed
end

-- https://github.com/Donearm/scripts/blob/master/lib/dirname.lua
function TTGUtils.dirname(str)
	if str:match(".-/.-") then
		local name = string.gsub(str, "(.*/)(.*)", "%1")
		return name
	else
		return ''
	end
end

function TTGUtils.getWatermarkDropdown()
	local watermarkDropdown = {}
	table.insert(watermarkDropdown,  {title = 'Managed by Lightroom', value = 'publisher'})
	table.insert(watermarkDropdown,  {title = 'Do not apply watermarks', value = 'none'})

	local watermarks,indexed = TTGUtils.getWatermarks()

	for id,watermark in pairs(watermarks) do
		table.insert(watermarkDropdown,  {title = watermark[2], value = watermark[1]})
	end

	return watermarkDropdown

end

function TTGUtils.isBacklight(apiURL)
	return string.match(apiURL, 'backlight/publisher');
end

function TTGUtils.isBacklight2(collectionSettings, properties)

	-- serverVersion attached to collection settings, only after first save
	--local inspected = TTGUtils.inspect(collectionSettings)
	--logger:debug('isBacklight2 collectionSettings: '..inspected)

	-- serverVersion attached never attached to properties -- no way to save this!
	--local inspected = TTGUtils.inspect(properties)
	--logger:debug('isBacklight2 properties: '..inspected)

    -- serverVersion attached to prefs, but this applies to all services
	--local prefs = import 'LrPrefs'.prefsForPlugin()
	--local inspected = TTGUtils.inspect(prefs)
	--logger:debug('isBacklight2 prefs: '..inspected)


	-- The below will work for all cases except for first load of adding album/set
	-- immediately after using CE4/Backlight service
    -- and possibly

    -- use collection settings, in case already saved collection
	local isBacklight2 = collectionSettings.serverVersion ~= nil and (string.match(collectionSettings.serverVersion, '^3.9') or string.match(collectionSettings.serverVersion, '^4') or string.match(collectionSettings.serverVersion, '^5') or string.match(collectionSettings.serverVersion, '^6'))

	if isBacklight2 then
		logger:debug('isBacklight2! (collectionSettings)')
		return isBacklight2
	end

	-- use prefs as fallback
	local prefs = import 'LrPrefs'.prefsForPlugin()
	isBacklight2 = prefs.serverVersion ~= nil and (string.match(prefs.serverVersion, '^3.9') or string.match(prefs.serverVersion, '^4') or string.match(prefs.serverVersion, '^5') or string.match(prefs.serverVersion, '^6'))

	if isBacklight2 then
		logger:debug('isBacklight2! (prefs)')
	else
		logger:debug('NOT isBacklight2 :/ (prefs)')
	end

	return isBacklight2
end


function TTGUtils.isBacklight3(collectionSettings, properties)

	local isBacklight3 = collectionSettings.serverVersion ~= nil and (string.match(collectionSettings.serverVersion, '^5') or string.match(collectionSettings.serverVersion, '^6'))

	if isBacklight3 then
		logger:debug('isBacklight3! (collectionSettings)')
		return isBacklight3
	end

	-- use prefs as fallback
	local prefs = import 'LrPrefs'.prefsForPlugin()
	isBacklight3 = prefs.serverVersion ~= nil and (string.match(prefs.serverVersion, '^5') or string.match(prefs.serverVersion, '^6'))

	if isBacklight3 then
		logger:debug('isBacklight3! (prefs)')
	else
		logger:debug('NOT isBacklight3 :/ (prefs)')
	end

	return isBacklight3
end